@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -43,7 +43,7 @@ module CustomFieldsHelper
   ]
 
   def render_custom_fields_tabs(types)
-    tabs = CUSTOM_FIELDS_TABS.select {|h| types.include?(h[:name]) }
+    tabs = CUSTOM_FIELDS_TABS.select {|h| types.include?(h[:name])}
     render_tabs tabs
   end
 
@@ -79,14 +79,13 @@ module CustomFieldsHelper
 
   # Return custom field html tag corresponding to its format
   def custom_field_tag(prefix, custom_value)
-    css = "#{custom_value.custom_field.field_format}_cf"
+    css = custom_value.custom_field.css_classes
     data = nil
     if custom_value.custom_field.full_text_formatting?
       css += ' wiki-edit'
       data = {
-        :auto_complete => true,
-        :issues_url => auto_complete_issues_path(:project_id => custom_value.customized.project, :q => '')
-      } if custom_value.customized && custom_value.customized.project
+        :auto_complete => true
+      }
     end
     custom_value.custom_field.format.edit_tag(
       self,
@@ -112,7 +111,8 @@ module CustomFieldsHelper
     content_tag(
       "label", content +
       (required ? " <span class=\"required\">*</span>".html_safe : ""),
-      :for => for_tag_id)
+      :for => for_tag_id,
+      :class => custom_value.customized && custom_value.customized.errors[custom_value.custom_field.name].present? ? 'error' : nil)
   end
 
   # Return custom field tag with its label tag
@@ -128,6 +128,14 @@ module CustomFieldsHelper
 
   # Returns the custom field tag for when bulk editing objects
   def custom_field_tag_for_bulk_edit(prefix, custom_field, objects=nil, value='')
+    css =  custom_field.css_classes
+    data = nil
+    if custom_field.full_text_formatting?
+      css += ' wiki-edit'
+      data = {
+        :auto_complete => true
+      }
+    end
     custom_field.format.bulk_edit_tag(
       self,
       custom_field_tag_id(prefix, custom_field),
@@ -135,7 +143,8 @@ module CustomFieldsHelper
       custom_field,
       objects,
       value,
-      :class => "#{custom_field.field_format}_cf")
+      :class => css,
+      :data => data)
   end
 
   # Returns custom field value tag
@@ -180,7 +189,7 @@ module CustomFieldsHelper
     api.array :custom_fields do
       custom_values.each do |custom_value|
         attrs = {:id => custom_value.custom_field_id, :name => custom_value.custom_field.name}
-        attrs.merge!(:multiple => true) if custom_value.custom_field.multiple?
+        attrs[:multiple] = true if custom_value.custom_field.multiple?
         api.custom_field attrs do
           if custom_value.value.is_a?(Array)
             api.array :value do
