@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,6 +20,7 @@
 module IssuesHelper
   include ApplicationHelper
   include Redmine::Export::PDF::IssuesPdfHelper
+  include IssueStatusesHelper
 
   def issue_list(issues, &block)
     ancestors = []
@@ -83,9 +84,6 @@ module IssuesHelper
     end
     s << '<div>'
     subject = h(issue.subject)
-    if issue.is_private?
-      subject = subject + ' ' + content_tag('span', l(:field_is_private), :class => 'badge badge-private private')
-    end
     s << content_tag('h3', subject)
     s << '</div>' * (ancestors.size + 1)
     s.html_safe
@@ -102,15 +100,17 @@ module IssuesHelper
       css << " idnt idnt-#{level}" if level > 0
       buttons =
         if manage_relations
-          link_to(l(:label_delete_link_to_subtask),
-                  issue_path(
-                    {:id => child.id, :issue => {:parent_issue_id => ''},
-                     :back_url => issue_path(issue.id), :no_flash => '1'}),
-                  :method => :put,
-                  :data => {:confirm => l(:text_are_you_sure)},
-                  :title => l(:label_delete_link_to_subtask),
-                  :class => 'icon-only icon-link-break'
-                  )
+          link_to(
+            l(:label_delete_link_to_subtask),
+            issue_path(
+              {:id => child.id, :issue => {:parent_issue_id => ''},
+               :back_url => issue_path(issue.id), :no_flash => '1'}
+            ),
+            :method => :put,
+            :data => {:confirm => l(:text_are_you_sure)},
+            :title => l(:label_delete_link_to_subtask),
+            :class => 'icon-only icon-link-break'
+          )
         else
           "".html_safe
         end
@@ -133,7 +133,7 @@ module IssuesHelper
                          (if child.disabled_core_fields.include?('done_ratio')
                             ''
                           else
-                             progress_bar(child.done_ratio)
+                            progress_bar(child.done_ratio)
                           end),
                          :class=> 'done_ratio') +
              content_tag('td', buttons, :class => 'buttons'),
@@ -143,6 +143,52 @@ module IssuesHelper
     s.html_safe
   end
 
+  # Renders descendants stats (total descendants (open - closed)) with query links
+  def render_descendants_stats(issue)
+    # Get issue descendants grouped by status type (open/closed) using a single query
+    subtasks_grouped = issue.descendants.visible.joins(:status).select(:is_closed, :id).group(:is_closed).reorder(:is_closed).count(:id)
+    # Cast keys to boolean in order to have consistent results between database types
+    subtasks_grouped.transform_keys! {|k| ActiveModel::Type::Boolean.new.cast(k)}
+
+    open_subtasks = subtasks_grouped[false].to_i
+    closed_subtasks = subtasks_grouped[true].to_i
+    all_subtasks = open_subtasks + closed_subtasks
+
+    return if all_subtasks == 0
+
+    all_block = content_tag(
+      'span',
+      link_to(all_subtasks, issues_path(parent_id: "~#{issue.id}", set_filter: true, status_id: '*')),
+      class: 'badge badge-issues-count'
+    )
+
+    closed_block = content_tag(
+      'span',
+      link_to_if(
+        closed_subtasks > 0,
+        l(:label_x_closed_issues_abbr, count: closed_subtasks),
+        issues_path(parent_id: "~#{issue.id}", set_filter: true, status_id: 'c')
+      ),
+      class: 'closed'
+    )
+
+    open_block = content_tag(
+      'span',
+      link_to_if(
+        open_subtasks > 0,
+        l(:label_x_open_issues_abbr, :count => open_subtasks),
+        issues_path(:parent_id => "~#{issue.id}", :set_filter => true, :status_id => 'o')
+      ),
+      class: 'open'
+    )
+
+    content_tag(
+      'span',
+      "#{all_block} (#{open_block} &#8212; #{closed_block})".html_safe,
+      :class => 'issues-stat'
+    )
+  end
+
   # Renders the list of related issues on the issue details view
   def render_issue_relations(issue, relations)
     manage_relations = User.current.allowed_to?(:manage_issue_relations, issue.project)
@@ -174,11 +220,12 @@ module IssuesHelper
                         false, :id => nil),
                       :class => 'checkbox') +
              content_tag('td',
-                         relation.to_s(@issue) {|other|
+                         relation.to_s(@issue) do |other|
                            link_to_issue(
                              other,
-                             :project => Setting.cross_project_issue_relations?)
-                         }.html_safe,
+                             :project => Setting.cross_project_issue_relations?
+                           )
+                         end.html_safe,
                          :class => 'subject') +
              content_tag('td', other_issue.status, :class => 'status') +
              content_tag('td', link_to_user(other_issue.assigned_to), :class => 'assigned_to') +
@@ -226,6 +273,7 @@ module IssuesHelper
 
   def issue_due_date_details(issue)
     return if issue&.due_date.nil?
+
     s = format_date(issue.due_date)
     s += " (#{due_date_distance_in_words(issue.due_date)})" unless issue.closed?
     s
@@ -233,11 +281,17 @@ module IssuesHelper
 
   # Returns a link for adding a new subtask to the given issue
   def link_to_new_subtask(issue)
+    link_to(l(:button_add), url_for_new_subtask(issue))
+  end
+
+  def url_for_new_subtask(issue)
     attrs = {
       :parent_issue_id => issue
     }
     attrs[:tracker_id] = issue.tracker unless issue.tracker.disabled_core_fields.include?('parent_issue_id')
-    link_to(l(:button_add), new_project_issue_path(issue.project, :issue => attrs, :back_url => issue_path(issue)))
+    params = {:issue => attrs}
+    params[:back_url] = issue_path(issue) if controller_name == 'issues' && action_name == 'show'
+    new_project_issue_path(issue.project, params)
   end
 
   def trackers_options_for_select(issue)
@@ -302,6 +356,7 @@ module IssuesHelper
   def render_half_width_custom_fields_rows(issue)
     values = issue.visible_custom_field_values.reject {|value| value.custom_field.full_width_layout?}
     return if values.empty?
+
     half = (values.size / 2.0).ceil
     issue_fields_rows do |rows|
       values.each_with_index do |value, i|
@@ -314,10 +369,12 @@ module IssuesHelper
   def render_full_width_custom_fields_rows(issue)
     values = issue.visible_custom_field_values.select {|value| value.custom_field.full_width_layout?}
     return if values.empty?
+
     s = ''.html_safe
     values.each_with_index do |value, i|
       attr_value_tag = custom_field_value_tag(value)
       next if attr_value_tag.blank?
+
       content =
         content_tag('hr') +
         content_tag('p', content_tag('strong', custom_field_name_tag(value.custom_field) )) +
@@ -364,10 +421,11 @@ module IssuesHelper
   # on the new issue form
   def users_for_new_issue_watchers(issue)
     users = issue.watcher_users.select{|u| u.status == User::STATUS_ACTIVE}
-    if issue.project.users.count <= 20
-      users = (users + issue.project.users.sort).uniq
+    assignable_watchers = issue.project.principals.assignable_watchers.limit(21)
+    if assignable_watchers.size <= 20
+      users += assignable_watchers.sort
     end
-    users
+    users.uniq
   end
 
   def email_issue_attributes(issue, user, html)
@@ -376,6 +434,7 @@ module IssuesHelper
       if issue.disabled_core_fields.grep(/^#{attribute}(_id)?$/).empty?
         attr_value = (issue.send attribute).to_s
         next if attr_value.blank?
+
         if html
           items << content_tag('strong', "#{l("field_#{attribute}")}: ") + attr_value
         else
@@ -386,6 +445,7 @@ module IssuesHelper
     issue.visible_custom_field_values(user).each do |value|
       cf_value = show_value(value, false)
       next if cf_value.blank?
+
       if html
         items << content_tag('strong', "#{value.custom_field.name}: ") + cf_value
       else
@@ -519,7 +579,7 @@ module IssuesHelper
       label = l(relation_type[:name]) if relation_type
     end
     call_hook(:helper_issues_show_detail_after_setting,
-              {:detail => detail, :label => label, :value => value, :old_value => old_value })
+              {:detail => detail, :label => label, :value => value, :old_value => old_value})
 
     label ||= detail.prop_key
     value ||= detail.value
@@ -551,7 +611,7 @@ module IssuesHelper
       unless no_html
         diff_link =
           link_to(
-            'diff',
+            l(:label_diff),
             diff_journal_url(detail.journal_id, :detail_id => detail.id,
                              :only_path => options[:only_path]),
             :title => l(:label_view_diff))
@@ -577,14 +637,16 @@ module IssuesHelper
   end
 
   # Find the name of an associated record stored in the field attribute
+  # For project, return the associated record only if is visible for the current User
   def find_name_by_reflection(field, id)
     return nil if id.blank?
+
     @detail_value_name_by_reflection ||= Hash.new do |hash, key|
       association = Issue.reflect_on_association(key.first.to_sym)
       name = nil
       if association
         record = association.klass.find_by_id(key.last)
-        if record
+        if (record && !record.is_a?(Project)) || (record.is_a?(Project) && record.visible?)
           name = record.name.force_encoding('UTF-8')
         end
       end
@@ -596,6 +658,7 @@ module IssuesHelper
   # Renders issue children recursively
   def render_api_issue_children(issue, api)
     return if issue.leaf?
+
     api.array :children do
       issue.children.each do |child|
         api.issue(:id => child.id) do
@@ -613,19 +676,62 @@ module IssuesHelper
     if @journals.present?
       journals_without_notes = @journals.select{|value| value.notes.blank?}
       journals_with_notes = @journals.reject{|value| value.notes.blank?}
-
-      tabs << {:name => 'history', :label => :label_history, :onclick => 'showIssueHistory("history", this.href)', :partial => 'issues/tabs/history', :locals => {:issue => @issue, :journals => @journals}}
-      tabs << {:name => 'notes', :label => :label_issue_history_notes, :onclick => 'showIssueHistory("notes", this.href)'} if journals_with_notes.any?
-      tabs << {:name => 'properties', :label => :label_issue_history_properties, :onclick => 'showIssueHistory("properties", this.href)'} if journals_without_notes.any?
+      tabs <<
+        {
+          :name => 'history',
+          :label => :label_history,
+          :onclick => 'showIssueHistory("history", this.href)',
+          :partial => 'issues/tabs/history',
+          :locals => {:issue => @issue, :journals => @journals}
+        }
+      if journals_with_notes.any?
+        tabs <<
+          {
+            :name => 'notes',
+            :label => :label_issue_history_notes,
+            :onclick => 'showIssueHistory("notes", this.href)'
+          }
+      end
+      if journals_without_notes.any?
+        tabs <<
+          {
+            :name => 'properties',
+            :label => :label_issue_history_properties,
+            :onclick => 'showIssueHistory("properties", this.href)'
+          }
+      end
+    end
+    if User.current.allowed_to?(:view_time_entries, @project) && @issue.spent_hours > 0
+      tabs <<
+        {
+          :name => 'time_entries',
+          :label => :label_time_entry_plural,
+          :remote => true,
+          :onclick =>
+            "getRemoteTab('time_entries', " \
+            "'#{tab_issue_path(@issue, :name => 'time_entries')}', " \
+            "'#{issue_path(@issue, :tab => 'time_entries')}')"
+        }
+    end
+    if @has_changesets
+      tabs <<
+        {
+          :name => 'changesets',
+          :label => :label_associated_revisions,
+          :remote => true,
+          :onclick =>
+            "getRemoteTab('changesets', " \
+            "'#{tab_issue_path(@issue, :name => 'changesets')}', " \
+            "'#{issue_path(@issue, :tab => 'changesets')}')"
+        }
     end
-    tabs << {:name => 'time_entries', :label => :label_time_entry_plural, :remote => true, :onclick => "getRemoteTab('time_entries', '#{tab_issue_path(@issue, :name => 'time_entries')}', '#{issue_path(@issue, :tab => 'time_entries')}')"} if User.current.allowed_to?(:view_time_entries, @project) && @issue.spent_hours > 0
-    tabs << {:name => 'changesets', :label => :label_associated_revisions, :remote => true, :onclick => "getRemoteTab('changesets', '#{tab_issue_path(@issue, :name => 'changesets')}', '#{issue_path(@issue, :tab => 'changesets')}')"} if @has_changesets
     tabs
   end
 
   def issue_history_default_tab
     # tab params overrides user default tab preference
     return params[:tab] if params[:tab].present?
+
     user_default_tab = User.current.pref.history_default_tab
 
     case user_default_tab
@@ -638,4 +744,13 @@ module IssuesHelper
     end
   end
 
+  def projects_for_select(issue)
+    if issue.parent_issue_id.present?
+      issue.allowed_target_projects_for_subtask(User.current)
+    elsif @project && issue.new_record? && !issue.copy?
+      issue.allowed_target_projects(User.current, 'tree')
+    else
+      issue.allowed_target_projects(User.current)
+    end
+  end
 end
