@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -54,7 +54,7 @@ module QueriesHelper
     end
     s = options_for_select([[]] + ungrouped)
     if grouped.present?
-      localized_grouped = grouped.map {|k,v| [k.is_a?(Symbol) ? l(k) : k.to_s, v]}
+      localized_grouped = grouped.map {|k, v| [k.is_a?(Symbol) ? l(k) : k.to_s, v]}
       s << grouped_options_for_select(localized_grouped)
     end
     s
@@ -92,7 +92,14 @@ module QueriesHelper
   def available_block_columns_tags(query)
     tags = ''.html_safe
     query.available_block_columns.each do |column|
-      tags << content_tag('label', check_box_tag('c[]', column.name.to_s, query.has_column?(column), :id => nil) + " #{column.caption}", :class => 'inline')
+      tags <<
+        content_tag(
+          'label',
+          check_box_tag(
+            'c[]', column.name.to_s,
+            query.has_column?(column), :id => nil
+          ) + " #{column.caption}", :class => 'inline'
+        )
     end
     tags
   end
@@ -101,18 +108,27 @@ module QueriesHelper
     tag_name = (options[:name] || 't') + '[]'
     tags = ''.html_safe
     query.available_totalable_columns.each do |column|
-      tags << content_tag('label', check_box_tag(tag_name, column.name.to_s, query.totalable_columns.include?(column), :id => nil) + " #{column.caption}", :class => 'inline')
+      tags <<
+        content_tag(
+          'label',
+          check_box_tag(
+            tag_name, column.name.to_s,
+            query.totalable_columns.include?(column), :id => nil
+          ) + " #{column.caption}", :class => 'inline'
+        )
     end
     tags << hidden_field_tag(tag_name, '')
     tags
   end
 
   def query_available_inline_columns_options(query)
-    (query.available_inline_columns - query.columns).reject(&:frozen?).collect {|column| [column.caption, column.name]}
+    (query.available_inline_columns - query.columns).
+      reject(&:frozen?).collect {|column| [column.caption, column.name]}
   end
 
   def query_selected_inline_columns_options(query)
-    (query.inline_columns & query.available_inline_columns).reject(&:frozen?).collect {|column| [column.caption, column.name]}
+    (query.inline_columns & query.available_inline_columns).
+      reject(&:frozen?).collect {|column| [column.caption, column.name]}
   end
 
   def render_query_columns_selection(query, options={})
@@ -158,6 +174,7 @@ module QueriesHelper
 
   def render_query_totals(query)
     return unless query.totalable_columns.present?
+
     totals = query.totalable_columns.map do |column|
       total_tag(column, query.total_for(column))
     end
@@ -167,7 +184,7 @@ module QueriesHelper
   def total_tag(column, value)
     label = content_tag('span', "#{column.caption}:")
     value =
-      if [:hours, :spent_hours, :total_spent_hours, :estimated_hours].include? column.name
+      if [:hours, :spent_hours, :total_spent_hours, :estimated_hours, :total_estimated_hours].include? column.name
         format_hours(value)
       else
         format_object(value)
@@ -192,13 +209,14 @@ module QueriesHelper
       sort_param = {param_key => query.sort_criteria.add(column.name, order).to_param}
       sort_param = {$1 => {$2 => sort_param.values.first}} while sort_param.keys.first.to_s =~ /^(.+)\[(.+)\]$/
       link_options = {
-          :title => l(:label_sort_by, "\"#{column.caption}\""),
-          :class => css
-        }
+        :title => l(:label_sort_by, "\"#{column.caption}\""),
+        :class => css
+      }
       if options[:sort_link_options]
         link_options.merge! options[:sort_link_options]
       end
-      content = link_to(
+      content =
+        link_to(
           column.caption,
           {:params => request.query_parameters.deep_merge(sort_param)},
           link_options
@@ -238,7 +256,7 @@ module QueriesHelper
         'span',
         value.to_s(item) {|other| link_to_issue(other, :subject => false, :tracker => false)}.html_safe,
         :class => value.css_classes_for(item))
-    when :hours, :estimated_hours
+    when :hours, :estimated_hours, :total_estimated_hours
       format_hours(value)
     when :spent_hours
       link_to_if(value > 0, format_hours(value), project_time_entries_path(item.project, :issue_id => "#{item.id}"))
@@ -306,18 +324,38 @@ module QueriesHelper
       scope = scope.or(klass.where(:project_id => @project)) if @project
       @query = scope.find(params[:query_id])
       raise ::Unauthorized unless @query.visible?
+
       @query.project = @project
       session[session_key] = {:id => @query.id, :project_id => @query.project_id} if use_session
-    elsif api_request? || params[:set_filter] || !use_session || session[session_key].nil? || session[session_key][:project_id] != (@project ? @project.id : nil)
+    elsif api_request? || params[:set_filter] || !use_session ||
+            session[session_key].nil? ||
+            session[session_key][:project_id] != (@project ? @project.id : nil)
       # Give it a name, required to be valid
       @query = klass.new(:name => "_", :project => @project)
       @query.build_from_params(params, options[:defaults])
-      session[session_key] = {:project_id => @query.project_id, :filters => @query.filters, :group_by => @query.group_by, :column_names => @query.column_names, :totalable_names => @query.totalable_names, :sort => @query.sort_criteria.to_a} if use_session
+      if use_session
+        session[session_key] = {
+          :project_id => @query.project_id,
+          :filters => @query.filters,
+          :group_by => @query.group_by,
+          :column_names => @query.column_names,
+          :totalable_names => @query.totalable_names,
+          :sort => @query.sort_criteria.to_a
+        }
+      end
     else
       # retrieve from session
       @query = nil
       @query = klass.find_by_id(session[session_key][:id]) if session[session_key][:id]
-      @query ||= klass.new(:name => "_", :filters => session[session_key][:filters], :group_by => session[session_key][:group_by], :column_names => session[session_key][:column_names], :totalable_names => session[session_key][:totalable_names], :sort_criteria => session[session_key][:sort])
+      @query ||=
+        klass.new(
+          :name => "_",
+          :filters => session[session_key][:filters],
+          :group_by => session[session_key][:group_by],
+          :column_names => session[session_key][:column_names],
+          :totalable_names => session[session_key][:totalable_names],
+          :sort_criteria => session[session_key][:sort]
+        )
       @query.project = @project
     end
     if params[:sort].present?
@@ -339,7 +377,15 @@ module QueriesHelper
         @query = IssueQuery.find_by_id(session_data[:id])
         return unless @query
       else
-        @query = IssueQuery.new(:name => "_", :filters => session_data[:filters], :group_by => session_data[:group_by], :column_names => session_data[:column_names], :totalable_names => session_data[:totalable_names], :sort_criteria => session[session_key][:sort])
+        @query =
+          IssueQuery.new(
+            :name => "_",
+            :filters => session_data[:filters],
+            :group_by => session_data[:group_by],
+            :column_names => session_data[:column_names],
+            :totalable_names => session_data[:totalable_names],
+            :sort_criteria => session[session_key][:sort]
+          )
       end
       if session_data.has_key?(:project_id)
         @query.project_id = session_data[:project_id]
@@ -395,6 +441,7 @@ module QueriesHelper
   # Renders a group of queries
   def query_links(title, queries)
     return '' if queries.empty?
+
     # links to #index on issues/show
     url_params =
       if controller_name == 'issues'
@@ -405,7 +452,7 @@ module QueriesHelper
     content_tag('h3', title) + "\n" +
       content_tag(
         'ul',
-        queries.collect {|query|
+        queries.collect do |query|
           css = +'query'
           clear_link = +''
           if query == @query
@@ -417,7 +464,7 @@ module QueriesHelper
                               url_params.merge(:query_id => query),
                               :class => css) +
                         clear_link.html_safe)
-        }.join("\n").html_safe,
+        end.join("\n").html_safe,
         :class => 'queries'
       ) + "\n"
   end
