@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -49,13 +49,14 @@ class Issue < ActiveRecord::Base
 
   acts_as_event :title => Proc.new {|o| "#{o.tracker.name} ##{o.id} (#{o.status}): #{o.subject}"},
                 :url => Proc.new {|o| {:controller => 'issues', :action => 'show', :id => o.id}},
-                :type => Proc.new {|o| 'issue' + (o.closed? ? '-closed' : '') }
+                :type => Proc.new {|o| 'issue' + (o.closed? ? '-closed' : '')}
 
-  acts_as_activity_provider :scope => preload(:project, :author, :tracker, :status),
+  acts_as_activity_provider :scope => proc {preload(:project, :author, :tracker, :status)},
                             :author_key => :author_id
 
   DONE_RATIO_OPTIONS = %w(issue_field issue_status)
 
+  attr_reader :transition_warning
   attr_writer :deleted_attachment_ids
   delegate :notes, :notes=, :private_notes, :private_notes=, :to => :current_journal, :allow_nil => true
 
@@ -71,45 +72,49 @@ class Issue < ActiveRecord::Base
   validates :due_date, :date => true
   validate :validate_issue, :validate_required_fields, :validate_permissions
 
-  scope :visible, lambda {|*args|
+  scope :visible, (lambda do |*args|
     joins(:project).
     where(Issue.visible_condition(args.shift || User.current, *args))
-  }
+  end)
 
-  scope :open, lambda {|*args|
+  scope :open, (lambda do |*args|
     is_closed = !args.empty? ? !args.first : false
     joins(:status).
     where(:issue_statuses => {:is_closed => is_closed})
-  }
+  end)
 
-  scope :recently_updated, lambda { order(:updated_on => :desc) }
-  scope :on_active_project, lambda {
+  scope :recently_updated, lambda {order(:updated_on => :desc)}
+  scope :on_active_project, (lambda do
     joins(:project).
     where(:projects => {:status => Project::STATUS_ACTIVE})
-  }
-  scope :fixed_version, lambda {|versions|
+  end)
+  scope :fixed_version, (lambda do |versions|
     ids = [versions].flatten.compact.map {|v| v.is_a?(Version) ? v.id : v}
     ids.any? ? where(:fixed_version_id => ids) : none
-  }
-  scope :assigned_to, lambda {|arg|
+  end)
+  scope :assigned_to, (lambda do |arg|
     arg = Array(arg).uniq
     ids = arg.map {|p| p.is_a?(Principal) ? p.id : p}
     ids += arg.select {|p| p.is_a?(User)}.map(&:group_ids).flatten.uniq
     ids.compact!
     ids.any? ? where(:assigned_to_id => ids) : none
-  }
-  scope :like, lambda {|q|
+  end)
+  scope :like, (lambda do |q|
     q = q.to_s
     if q.present?
       where("LOWER(#{table_name}.subject) LIKE LOWER(?)", "%#{q}%")
     end
-  }
+  end)
 
   before_validation :default_assign, on: :create
   before_validation :clear_disabled_fields
   before_save :close_duplicates, :update_done_ratio_from_issue_status,
               :force_updated_on_change, :update_closed_on
-  after_save {|issue| issue.send :after_project_change if !issue.saved_change_to_id? && issue.saved_change_to_project_id?}
+  after_save do |issue|
+    if !issue.saved_change_to_id? && issue.saved_change_to_project_id?
+      issue.send :after_project_change
+    end
+  end
   after_save :reschedule_following_issues, :update_nested_set_attributes,
              :update_parent_attributes, :delete_selected_attachments, :create_journal
   # Should be after_create but would be called before previous after_save callbacks
@@ -124,15 +129,18 @@ class Issue < ActiveRecord::Base
         if user.id && user.logged?
           case role.issues_visibility
           when 'all'
-              '1=1'
+            '1=1'
           when 'default'
             user_ids = [user.id] + user.groups.pluck(:id).compact
-            "(#{table_name}.is_private = #{connection.quoted_false} OR #{table_name}.author_id = #{user.id} OR #{table_name}.assigned_to_id IN (#{user_ids.join(',')}))"
+            "(#{table_name}.is_private = #{connection.quoted_false} " \
+              "OR #{table_name}.author_id = #{user.id} " \
+              "OR #{table_name}.assigned_to_id IN (#{user_ids.join(',')}))"
           when 'own'
             user_ids = [user.id] + user.groups.pluck(:id).compact
-            "(#{table_name}.author_id = #{user.id} OR #{table_name}.assigned_to_id IN (#{user_ids.join(',')}))"
+            "(#{table_name}.author_id = #{user.id} OR " \
+              "#{table_name}.assigned_to_id IN (#{user_ids.join(',')}))"
           else
-              '1=0'
+            '1=0'
           end
         else
           "(#{table_name}.is_private = #{connection.quoted_false})"
@@ -272,8 +280,16 @@ class Issue < ActiveRecord::Base
   # Copies attributes from another issue, arg can be an id or an Issue
   def copy_from(arg, options={})
     issue = arg.is_a?(Issue) ? arg : Issue.visible.find(arg)
-    self.attributes = issue.attributes.dup.except("id", "root_id", "parent_id", "lft", "rgt", "created_on", "updated_on", "status_id", "closed_on")
-    self.custom_field_values = issue.custom_field_values.inject({}) {|h,v| h[v.custom_field_id] = v.value; h}
+    self.attributes =
+      issue.attributes.dup.except(
+        "id", "root_id", "parent_id", "lft", "rgt",
+        "created_on", "updated_on", "status_id", "closed_on"
+      )
+    self.custom_field_values =
+      issue.custom_field_values.inject({}) do |h, v|
+        h[v.custom_field_id] = v.value
+        h
+      end
     if options[:keep_status]
       self.status = issue.status
     end
@@ -435,6 +451,7 @@ class Issue < ActiveRecord::Base
   # Overrides assign_attributes so that project and tracker get assigned first
   def assign_attributes(new_attributes, *args)
     return if new_attributes.nil?
+
     attrs = new_attributes.dup
     attrs.stringify_keys!
 
@@ -471,7 +488,6 @@ class Issue < ActiveRecord::Base
     'custom_field_values',
     'custom_fields',
     'lock_version',
-    'notes',
     :if => lambda {|issue, user| issue.new_record? || issue.attributes_editable?(user)})
   safe_attributes(
     'notes',
@@ -484,16 +500,16 @@ class Issue < ActiveRecord::Base
     :if => lambda {|issue, user| issue.new_record? && user.allowed_to?(:add_issue_watchers, issue.project)})
   safe_attributes(
     'is_private',
-    :if => lambda {|issue, user|
+    :if => lambda do |issue, user|
       user.allowed_to?(:set_issues_private, issue.project) ||
         (issue.author_id == user.id && user.allowed_to?(:set_own_issues_private, issue.project))
-    })
+    end)
   safe_attributes(
     'parent_issue_id',
-    :if => lambda {|issue, user|
+    :if => lambda do |issue, user|
       (issue.new_record? || issue.attributes_editable?(user)) &&
         user.allowed_to?(:manage_subtasks, issue.project)
-    })
+    end)
   safe_attributes(
     'deleted_attachment_ids',
     :if => lambda {|issue, user| issue.attachments_deletable?(user)})
@@ -661,7 +677,11 @@ class Issue < ActiveRecord::Base
     return {} if roles.empty?
 
     result = {}
-    workflow_permissions = WorkflowPermission.where(:tracker_id => tracker_id, :old_status_id => status_id, :role_id => roles.map(&:id)).to_a
+    workflow_permissions =
+      WorkflowPermission.where(
+        :tracker_id => tracker_id, :old_status_id => status_id,
+        :role_id => roles.map(&:id)
+      ).to_a
     if workflow_permissions.any?
       workflow_rules = workflow_permissions.inject({}) do |h, wp|
         h[wp.field_name] ||= {}
@@ -669,7 +689,9 @@ class Issue < ActiveRecord::Base
         h
       end
       fields_with_roles = {}
-      IssueCustomField.where(:visible => false).joins(:roles).pluck(:id, "role_id").each do |field_id, role_id|
+      IssueCustomField.where(:visible => false).
+        joins(:roles).pluck(:id, "role_id").
+          each do |field_id, role_id|
         fields_with_roles[field_id] ||= []
         fields_with_roles[field_id] << role_id
       end
@@ -684,6 +706,7 @@ class Issue < ActiveRecord::Base
       end
       workflow_rules.each do |attr, rules|
         next if rules.size < roles.size
+
         uniq_rules = rules.values.uniq
         if uniq_rules.size == 1
           result[attr] = uniq_rules.first
@@ -722,7 +745,7 @@ class Issue < ActiveRecord::Base
       errors.add :start_date, :earlier_than_minimum_start_date, :date => format_date(soonest_start)
     end
 
-    if fixed_version
+    if project && fixed_version
       if !assignable_versions.include?(fixed_version)
         errors.add :fixed_version_id, :inclusion
       elsif reopening? && fixed_version.closed?
@@ -737,7 +760,7 @@ class Issue < ActiveRecord::Base
       end
     end
 
-    if assigned_to_id_changed? && assigned_to_id.present?
+    if project && assigned_to_id_changed? && assigned_to_id.present?
       unless assignable_users.include?(assigned_to)
         errors.add :assigned_to_id, :invalid
       end
@@ -751,7 +774,12 @@ class Issue < ActiveRecord::Base
         errors.add :parent_issue_id, :invalid
       elsif (@parent_issue != parent) && (
           self.would_reschedule?(@parent_issue) ||
-          @parent_issue.self_and_ancestors.any? {|a| a.relations_from.any? {|r| r.relation_type == IssueRelation::TYPE_PRECEDES && r.issue_to.would_reschedule?(self)}}
+          @parent_issue.self_and_ancestors.any? do |a|
+            a.relations_from.any? do |r|
+              r.relation_type == IssueRelation::TYPE_PRECEDES &&
+                 r.issue_to.would_reschedule?(self)
+            end
+          end
         )
         errors.add :parent_issue_id, :invalid
       elsif !closed? && @parent_issue.closed?
@@ -775,14 +803,15 @@ class Issue < ActiveRecord::Base
     required_attribute_names(user).each do |attribute|
       if /^\d+$/.match?(attribute)
         attribute = attribute.to_i
-        v = custom_field_values.detect {|v| v.custom_field_id == attribute }
+        v = custom_field_values.detect {|v| v.custom_field_id == attribute}
         if v && Array(v.value).detect(&:present?).nil?
-          errors.add :base, v.custom_field.name + ' ' + l('activerecord.errors.messages.blank')
+          errors.add(v.custom_field.name, l('activerecord.errors.messages.blank'))
         end
       else
         if respond_to?(attribute) && send(attribute).blank? && !disabled_core_fields.include?(attribute)
           next if attribute == 'category_id' && project.try(:issue_categories).blank?
           next if attribute == 'fixed_version_id' && assignable_versions.blank?
+
           errors.add attribute, :blank
         end
       end
@@ -865,7 +894,7 @@ class Issue < ActiveRecord::Base
       preload(:user => :email_address).
       reorder(:created_on, :id).to_a
 
-    result.each_with_index {|j,i| j.indice = i+1}
+    result.each_with_index {|j, i| j.indice = i + 1}
 
     unless user.allowed_to?(:view_private_notes, project)
       result.select! do |journal|
@@ -926,6 +955,7 @@ class Issue < ActiveRecord::Base
   # Is the amount of work done less than it should for the due date
   def behind_schedule?
     return false if start_date.nil? || due_date.nil?
+
     done_date = start_date + ((due_date - start_date + 1) * done_ratio / 100).floor
     return done_date <= User.current.today
   end
@@ -937,6 +967,8 @@ class Issue < ActiveRecord::Base
 
   # Users the issue can be assigned to
   def assignable_users
+    return [] if project.nil?
+
     users = project.assignable_users(tracker).to_a
     users << author if author && author.active?
     if assigned_to_id_was.present? && assignee = Principal.find_by_id(assigned_to_id_was)
@@ -948,6 +980,7 @@ class Issue < ActiveRecord::Base
   # Versions that the issue can be assigned to
   def assignable_versions
     return @assignable_versions if @assignable_versions
+    return [] if project.nil?
 
     versions = project.shared_versions.open.to_a
     if fixed_version
@@ -969,6 +1002,28 @@ class Issue < ActiveRecord::Base
     !relations_to.detect {|ir| ir.relation_type == 'blocks' && !ir.issue_from.closed?}.nil?
   end
 
+  # Returns true if this issue can be closed and if not, returns false and populates the reason
+  def closable?
+    if descendants.open.any?
+      @transition_warning = l(:notice_issue_not_closable_by_open_tasks)
+      return false
+    end
+    if blocked?
+      @transition_warning = l(:notice_issue_not_closable_by_blocking_issue)
+      return false
+    end
+    return true
+  end
+
+  # Returns true if this issue can be reopen and if not, returns false and populates the reason
+  def reopenable?
+    if ancestors.open(false).any?
+      @transition_warning = l(:notice_issue_not_reopenable_by_closed_parent_issue)
+      return false
+    end
+    return true
+  end
+
   # Returns the default status of the issue based on its tracker
   # Returns nil if tracker is nil
   def default_status
@@ -1008,11 +1063,11 @@ class Issue < ActiveRecord::Base
     statuses << default_status if include_default || (new_record? && statuses.empty?)
 
     statuses = statuses.compact.uniq.sort
-    if blocked? || descendants.open.any?
+    unless closable?
       # cannot close a blocked issue or a parent with open subtasks
       statuses.reject!(&:is_closed?)
     end
-    if ancestors.open(false).any?
+    unless reopenable?
       # cannot reopen a subtask of a closed parent
       statuses.select!(&:is_closed?)
     end
@@ -1027,7 +1082,13 @@ class Issue < ActiveRecord::Base
   # Returns the previous assignee whenever we're before the save
   # or in after_* callbacks
   def previous_assignee
-    if previous_assigned_to_id = assigned_to_id_change_to_be_saved.nil? ? assigned_to_id_before_last_save : assigned_to_id_in_database
+    previous_assigned_to_id =
+      if assigned_to_id_change_to_be_saved.nil?
+        assigned_to_id_before_last_save
+      else
+        assigned_to_id_in_database
+      end
+    if previous_assigned_to_id
       Principal.find_by_id(previous_assigned_to_id)
     end
   end
@@ -1042,6 +1103,7 @@ class Issue < ActiveRecord::Base
     notified = notified.select {|u| u.active? && u.notify_about?(self)}
 
     notified += project.notified_users
+    notified += project.users.preload(:preference).select(&:notify_about_high_priority_issues?) if priority.high?
     notified.uniq!
     # Remove users that can not view the issue
     notified.reject! {|user| !visible?(user)}
@@ -1107,9 +1169,15 @@ class Issue < ActiveRecord::Base
   # Preloads relations for a collection of issues
   def self.load_relations(issues)
     if issues.any?
-      relations = IssueRelation.where("issue_from_id IN (:ids) OR issue_to_id IN (:ids)", :ids => issues.map(&:id)).all
+      relations =
+        IssueRelation.where(
+          "issue_from_id IN (:ids) OR issue_to_id IN (:ids)", :ids => issues.map(&:id)
+        ).all
       issues.each do |issue|
-        issue.instance_variable_set "@relations", relations.select {|r| r.issue_from_id == issue.id || r.issue_to_id == issue.id}
+        issue.instance_variable_set(
+          "@relations",
+          relations.select {|r| r.issue_from_id == issue.id || r.issue_to_id == issue.id}
+        )
       end
     end
   end
@@ -1160,10 +1228,11 @@ class Issue < ActiveRecord::Base
 
   # Returns a scope of the given issues and their descendants
   def self.self_and_descendants(issues)
-    Issue.joins("JOIN #{Issue.table_name} ancestors" +
-        " ON ancestors.root_id = #{Issue.table_name}.root_id" +
-        " AND ancestors.lft <= #{Issue.table_name}.lft AND ancestors.rgt >= #{Issue.table_name}.rgt"
-      ).
+    Issue.joins(
+      "JOIN #{Issue.table_name} ancestors" +
+      " ON ancestors.root_id = #{Issue.table_name}.root_id" +
+      " AND ancestors.lft <= #{Issue.table_name}.lft AND ancestors.rgt >= #{Issue.table_name}.rgt"
+    ).
       where(:ancestors => {:id => issues.map(&:id)})
   end
 
@@ -1216,10 +1285,14 @@ class Issue < ActiveRecord::Base
     all = [self]
     last = [self]
     while last.any?
-      current = last.map {|i| i.relations_from.where(:relation_type => IssueRelation::TYPE_BLOCKS).map(&:issue_to)}.flatten.uniq
+      current =
+        last.map do |i|
+          i.relations_from.where(:relation_type => IssueRelation::TYPE_BLOCKS).map(&:issue_to)
+        end.flatten.uniq
       current -= last
       current -= all
       return true if current.include?(other)
+
       last = current
       all += last
     end
@@ -1231,14 +1304,15 @@ class Issue < ActiveRecord::Base
     all = [self]
     last = [self]
     while last.any?
-      current = last.map {|i|
+      current = last.map do |i|
         i.relations_from.where(:relation_type => IssueRelation::TYPE_PRECEDES).map(&:issue_to) +
         i.leaves.to_a +
         i.ancestors.map {|a| a.relations_from.where(:relation_type => IssueRelation::TYPE_PRECEDES).map(&:issue_to)}
-      }.flatten.uniq
+      end.flatten.uniq
       current -= last
       current -= all
       return true if current.include?(other)
+
       last = current
       all += last
     end
@@ -1296,6 +1370,7 @@ class Issue < ActiveRecord::Base
   # If the issue is a parent task, this is done by rescheduling its subtasks.
   def reschedule_on!(date, journal=nil)
     return if date.nil?
+
     if leaf? || !dates_derived?
       if start_date.nil? || start_date != date
         if start_date && start_date > date
@@ -1363,6 +1438,7 @@ class Issue < ActiveRecord::Base
     s << ' child' if child?
     s << ' parent' unless leaf?
     s << ' private' if is_private?
+    s << ' behind-schedule' if behind_schedule?
     if user.logged?
       s << ' created-by-me' if author_id == user.id
       s << ' assigned-to-me' if assigned_to_id == user.id
@@ -1382,10 +1458,11 @@ class Issue < ActiveRecord::Base
   def self.update_versions_from_hierarchy_change(project)
     moved_project_ids = project.self_and_descendants.reload.pluck(:id)
     # Update issues of the moved projects and issues assigned to a version of a moved project
-    Issue.update_versions(
-            ["#{Version.table_name}.project_id IN (?) OR #{Issue.table_name}.project_id IN (?)",
-             moved_project_ids, moved_project_ids]
-          )
+    Issue.
+      update_versions(
+        ["#{Version.table_name}.project_id IN (?) OR #{Issue.table_name}.project_id IN (?)",
+         moved_project_ids, moved_project_ids]
+      )
   end
 
   def parent_issue_id=(arg)
@@ -1439,6 +1516,7 @@ class Issue < ActiveRecord::Base
     if project.nil?
       return Issue
     end
+
     case scope
     when 'all', 'system'
       Issue
@@ -1446,8 +1524,13 @@ class Issue < ActiveRecord::Base
       Issue.joins(:project).where("(#{Project.table_name}.lft >= :lft AND #{Project.table_name}.rgt <= :rgt)",
                                   :lft => project.root.lft, :rgt => project.root.rgt)
     when 'hierarchy'
-      Issue.joins(:project).where("(#{Project.table_name}.lft >= :lft AND #{Project.table_name}.rgt <= :rgt) OR (#{Project.table_name}.lft < :lft AND #{Project.table_name}.rgt > :rgt)",
-                                  :lft => project.lft, :rgt => project.rgt)
+      Issue.joins(:project).
+        where(
+          "(#{Project.table_name}.lft >= :lft AND " \
+            "#{Project.table_name}.rgt <= :rgt) OR " \
+            "(#{Project.table_name}.lft < :lft AND #{Project.table_name}.rgt > :rgt)",
+          :lft => project.lft, :rgt => project.rgt
+        )
     when 'descendants'
       Issue.joins(:project).where("(#{Project.table_name}.lft >= :lft AND #{Project.table_name}.rgt <= :rgt)",
                                   :lft => project.lft, :rgt => project.rgt)
@@ -1513,29 +1596,38 @@ class Issue < ActiveRecord::Base
       end
   end
 
-  # Returns a scope of projects that user can assign the issue to
-  def allowed_target_projects(user=User.current, context=nil)
-    if new_record? && context.is_a?(Project) && !copy?
-      current_project = context.self_and_descendants
-    elsif new_record?
-      current_project = nil
-    else
-      current_project = project
+  # Returns a scope of projects that user can assign the subtask
+  def allowed_target_projects_for_subtask(user=User.current)
+    if parent_issue_id.present?
+      scope = filter_projects_scope(Setting.cross_project_subtasks)
     end
 
-    self.class.allowed_target_projects(user, current_project)
+    self.class.allowed_target_projects(user, project, scope)
+  end
+
+  # Returns a scope of projects that user can assign the issue to
+  def allowed_target_projects(user=User.current, scope=nil)
+    current_project = new_record? ? nil : project
+    if scope
+      scope = filter_projects_scope(scope)
+    end
+
+    self.class.allowed_target_projects(user, current_project, scope)
   end
 
   # Returns a scope of projects that user can assign issues to
   # If current_project is given, it will be included in the scope
-  def self.allowed_target_projects(user=User.current, current_project=nil)
+  def self.allowed_target_projects(user=User.current, current_project=nil, scope=nil)
     condition = Project.allowed_to_condition(user, :add_issues)
-    if current_project.is_a?(Project)
+    if current_project
       condition = ["(#{condition}) OR #{Project.table_name}.id = ?", current_project.id]
-    elsif current_project
-      condition = ["(#{condition}) AND #{Project.table_name}.id IN (?)", current_project.map(&:id)]
     end
-    Project.where(condition).having_trackers
+
+    if scope.nil?
+      scope = Project
+    end
+
+    scope.where(condition).having_trackers
   end
 
   # Returns a scope of trackers that user can assign the issue to
@@ -1575,7 +1667,10 @@ class Issue < ActiveRecord::Base
       true
     else
       roles = user.roles_for_project(project).select {|r| r.has_permission?(permission)}
-      roles.any? {|r| r.permissions_all_trackers?(permission) || r.permissions_tracker_ids?(permission, tracker_id)}
+      roles.any? do |r|
+        r.permissions_all_trackers?(permission) ||
+          r.permissions_tracker_ids?(permission, tracker_id)
+      end
     end
   end
 
@@ -1592,10 +1687,16 @@ class Issue < ActiveRecord::Base
     # Move subtasks that were in the same project
     children.each do |child|
       next unless child.project_id == project_id_before_last_save
+
       # Change project and keep project
       child.send :project=, project, true
       unless child.save
-        errors.add :base, l(:error_move_of_child_not_possible, :child => "##{child.id}", :errors => child.errors.full_messages.join(", "))
+        errors.add(
+          :base,
+          l(:error_move_of_child_not_possible,
+            :child => "##{child.id}",
+            :errors => child.errors.full_messages.join(", "))
+        )
         raise ActiveRecord::Rollback
       end
     end
@@ -1607,13 +1708,22 @@ class Issue < ActiveRecord::Base
   def after_create_from_copy
     return unless copy? && !@after_create_from_copy_handled
 
-    if (@copied_from.project_id == project_id || Setting.cross_project_issue_relations?) && @copy_options[:link] != false
+    if (@copied_from.project_id == project_id ||
+          Setting.cross_project_issue_relations?) &&
+        @copy_options[:link] != false
       if @current_journal
         @copied_from.init_journal(@current_journal.user)
       end
-      relation = IssueRelation.new(:issue_from => @copied_from, :issue_to => self, :relation_type => IssueRelation::TYPE_COPIED_TO)
+      relation =
+        IssueRelation.new(:issue_from => @copied_from, :issue_to => self,
+                          :relation_type => IssueRelation::TYPE_COPIED_TO)
       unless relation.save
-        logger.error "Could not create relation while copying ##{@copied_from.id} to ##{id} due to validation errors: #{relation.errors.full_messages.join(', ')}" if logger
+        if logger
+          logger.error(
+            "Could not create relation while copying ##{@copied_from.id} to ##{id} " \
+              "due to validation errors: #{relation.errors.full_messages.join(', ')}"
+          )
+        end
       end
     end
 
@@ -1625,9 +1735,15 @@ class Issue < ActiveRecord::Base
         next if child == self
         # Do not copy subtasks of issues that were not copied
         next unless copied_issue_ids[child.parent_id]
+
         # Do not copy subtasks that are not visible to avoid potential disclosure of private data
         unless child.visible?
-          logger.error "Subtask ##{child.id} was not copied during ##{@copied_from.id} copy because it is not visible to the current user" if logger
+          if logger
+            logger.error(
+              "Subtask ##{child.id} was not copied during ##{@copied_from.id} copy " \
+                "because it is not visible to the current user"
+            )
+          end
           next
         end
         copy = Issue.new.copy_from(child, copy_options)
@@ -1637,10 +1753,21 @@ class Issue < ActiveRecord::Base
         copy.author = author
         copy.project = project
         copy.parent_issue_id = copied_issue_ids[child.parent_id]
-        copy.fixed_version_id = nil unless child.fixed_version.present? && child.fixed_version.status == 'open'
-        copy.assigned_to = nil unless child.assigned_to_id.present? && child.assigned_to.status == User::STATUS_ACTIVE
+        unless child.fixed_version.present? && child.fixed_version.status == 'open'
+          copy.fixed_version_id = nil
+        end
+        unless child.assigned_to_id.present? &&
+                 child.assigned_to.status == User::STATUS_ACTIVE
+          copy.assigned_to = nil
+        end
         unless copy.save
-          logger.error "Could not copy subtask ##{child.id} while copying ##{@copied_from.id} to ##{id} due to validation errors: #{copy.errors.full_messages.join(', ')}" if logger
+          if logger
+            logger.error(
+              "Could not copy subtask ##{child.id} " \
+                "while copying ##{@copied_from.id} to ##{id} due to validation errors: " \
+                "#{copy.errors.full_messages.join(', ')}"
+            )
+          end
           next
         end
         copied_issue_ids[child.id] = copy.id
@@ -1681,7 +1808,8 @@ class Issue < ActiveRecord::Base
       if p.priority_derived?
         # priority = highest priority of open children
         # priority is left unchanged if all children are closed and there's no default priority defined
-        if priority_position = p.children.open.joins(:priority).maximum("#{IssuePriority.table_name}.position")
+        if priority_position =
+             p.children.open.joins(:priority).maximum("#{IssuePriority.table_name}.position")
           p.priority = IssuePriority.find_by_position(priority_position)
         elsif default_priority = IssuePriority.default
           p.priority = default_priority
@@ -1704,16 +1832,18 @@ class Issue < ActiveRecord::Base
           if children.any?
             child_with_total_estimated_hours = children.select {|c| c.total_estimated_hours.to_f > 0.0}
             if child_with_total_estimated_hours.any?
-              average = child_with_total_estimated_hours.map(&:total_estimated_hours).sum.to_f / child_with_total_estimated_hours.count
+              average =
+                child_with_total_estimated_hours.sum(&:total_estimated_hours).to_d /
+                  child_with_total_estimated_hours.count
             else
-              average = 1.0
+              average = 1.0.to_d
             end
-            done = children.map {|c|
-              estimated = c.total_estimated_hours.to_f
+            done = children.sum do |c|
+              estimated = (c.total_estimated_hours || 0.0).to_d
               estimated = average unless estimated > 0.0
               ratio = c.closed? ? 100 : (c.done_ratio || 0)
               estimated * ratio
-            }.sum
+            end
             progress = done / (average * children.count)
             p.done_ratio = progress.floor
           end
@@ -1734,10 +1864,11 @@ class Issue < ActiveRecord::Base
       # a different project and that is not systemwide shared
       Issue.joins(:project, :fixed_version).
         where("#{Issue.table_name}.fixed_version_id IS NOT NULL" +
-          " AND #{Issue.table_name}.project_id <> #{Version.table_name}.project_id" +
-          " AND #{Version.table_name}.sharing <> 'system'").
+          " AND #{Issue.table_name}.project_id <> #{::Version.table_name}.project_id" +
+          " AND #{::Version.table_name}.sharing <> 'system'").
         where(conditions).each do |issue|
         next if issue.project.nil? || issue.fixed_version.nil?
+
         unless issue.project.shared_versions.include?(issue.fixed_version)
           issue.init_journal(User.current)
           issue.fixed_version = nil
@@ -1756,7 +1887,7 @@ class Issue < ActiveRecord::Base
 
   # Callback on file attachment
   def attachment_added(attachment)
-    if current_journal && !attachment.new_record?
+    if current_journal && !attachment.new_record? && !copy?
       current_journal.journalize_attachment(attachment, :added)
     end
   end
@@ -1813,6 +1944,7 @@ class Issue < ActiveRecord::Base
         duplicate.reload
         # Don't re-close it if it's already closed
         next if duplicate.closed?
+
         # Same user and notes
         if @current_journal
           duplicate.init_journal(@current_journal.user, @current_journal.notes)
@@ -1865,4 +1997,21 @@ class Issue < ActiveRecord::Base
       self.done_ratio ||= 0
     end
   end
+
+  def filter_projects_scope(scope=nil)
+    case scope
+    when 'system'
+      Project
+    when 'tree'
+      project.root.self_and_descendants
+    when 'hierarchy'
+      project.hierarchy
+    when 'descendants'
+      project.self_and_descendants
+    when ''
+      Project.where(:id => project.id)
+    else
+      Project
+    end
+  end
 end
