@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -34,7 +34,7 @@ class Project < ActiveRecord::Base
   has_many :memberships, :class_name => 'Member', :inverse_of => :project
   # Memberships of active users only
   has_many :members,
-           lambda { joins(:principal).where(:users => {:type => 'User', :status => Principal::STATUS_ACTIVE}) }
+           lambda {joins(:principal).where(:users => {:type => 'User', :status => Principal::STATUS_ACTIVE})}
   has_many :enabled_modules, :dependent => :delete_all
   has_and_belongs_to_many :trackers, lambda {order(:position)}
   has_many :issues, :dependent => :destroy
@@ -64,49 +64,55 @@ class Project < ActiveRecord::Base
                      :delete_permission => :manage_files
 
   acts_as_customizable
-  acts_as_searchable :columns => ['name', 'identifier', 'description'], :project_key => "#{Project.table_name}.id", :permission => nil
-  acts_as_event :title => Proc.new {|o| "#{l(:label_project)}: #{o.name}"},
-                :url => Proc.new {|o| {:controller => 'projects', :action => 'show', :id => o}},
+  acts_as_searchable :columns => ['name', 'identifier', 'description'],
+                     :project_key => "#{Project.table_name}.id",
+                     :permission => nil
+  acts_as_event :title => proc {|o| "#{l(:label_project)}: #{o.name}"},
+                :url => proc {|o| {:controller => 'projects', :action => 'show', :id => o}},
                 :author => nil
 
   validates_presence_of :name, :identifier
-  validates_uniqueness_of :identifier, :if => Proc.new {|p| p.identifier_changed?}
+  validates_uniqueness_of :identifier, :if => proc {|p| p.identifier_changed?}
   validates_length_of :name, :maximum => 255
   validates_length_of :homepage, :maximum => 255
   validates_length_of :identifier, :maximum => IDENTIFIER_MAX_LENGTH
   # downcase letters, digits, dashes but not digits only
-  validates_format_of :identifier, :with => /\A(?!\d+$)[a-z0-9\-_]*\z/, :if => Proc.new { |p| p.identifier_changed? }
+  validates_format_of :identifier, :with => /\A(?!\d+$)[a-z0-9\-_]*\z/,
+                      :if => proc {|p| p.identifier_changed?}
   # reserved words
-  validates_exclusion_of :identifier, :in => %w( new )
+  validates_exclusion_of :identifier, :in => %w(new)
   validate :validate_parent
 
-  after_save :update_inherited_members, :if => Proc.new {|project| project.saved_change_to_inherit_members?}
-  after_save :remove_inherited_member_roles, :add_inherited_member_roles, :if => Proc.new {|project| project.saved_change_to_parent_id?}
-  after_update :update_versions_from_hierarchy_change, :if => Proc.new {|project| project.saved_change_to_parent_id?}
+  after_save :update_inherited_members,
+             :if => proc {|project| project.saved_change_to_inherit_members?}
+  after_save :remove_inherited_member_roles, :add_inherited_member_roles,
+             :if => proc {|project| project.saved_change_to_parent_id?}
+  after_update :update_versions_from_hierarchy_change,
+               :if => proc {|project| project.saved_change_to_parent_id?}
   before_destroy :delete_all_members
 
-  scope :has_module, lambda {|mod|
+  scope :has_module, (lambda do |mod|
     where("#{Project.table_name}.id IN (SELECT em.project_id FROM #{EnabledModule.table_name} em WHERE em.name=?)", mod.to_s)
-  }
-  scope :active, lambda { where(:status => STATUS_ACTIVE) }
-  scope :status, lambda {|arg| where(arg.blank? ? nil : {:status => arg.to_i}) }
-  scope :all_public, lambda { where(:is_public => true) }
-  scope :visible, lambda {|*args| where(Project.visible_condition(args.shift || User.current, *args)) }
-  scope :allowed_to, lambda {|*args|
+  end)
+  scope :active, lambda {where(:status => STATUS_ACTIVE)}
+  scope :status, lambda {|arg| where(arg.blank? ? nil : {:status => arg.to_i})}
+  scope :all_public, lambda {where(:is_public => true)}
+  scope :visible, lambda {|*args| where(Project.visible_condition(args.shift || User.current, *args))}
+  scope :allowed_to, (lambda do |*args|
     user = args.first.is_a?(Symbol) ? User.current : args.shift
     permission = args.shift
     where(Project.allowed_to_condition(user, permission, *args))
-  }
-  scope :like, lambda {|arg|
+  end)
+  scope :like, (lambda do |arg|
     if arg.present?
       pattern = "%#{arg.to_s.strip}%"
       where("LOWER(identifier) LIKE LOWER(:p) OR LOWER(name) LIKE LOWER(:p)", :p => pattern)
     end
-  }
+  end)
   scope :sorted, lambda {order(:lft)}
-  scope :having_trackers, lambda {
+  scope :having_trackers, (lambda do
     where("#{Project.table_name}.id IN (SELECT DISTINCT project_id FROM #{table_name_prefix}projects_trackers#{table_name_suffix})")
-  }
+  end)
 
   def initialize(attributes=nil, *args)
     super
@@ -172,10 +178,18 @@ class Project < ActiveRecord::Base
   # * :member => true                   limit the condition to the user projects
   def self.allowed_to_condition(user, permission, options={})
     perm = Redmine::AccessControl.permission(permission)
-    base_statement = (perm && perm.read? ? "#{Project.table_name}.status <> #{Project::STATUS_ARCHIVED}" : "#{Project.table_name}.status = #{Project::STATUS_ACTIVE}")
+    base_statement =
+      if perm && perm.read?
+        "#{Project.table_name}.status <> #{Project::STATUS_ARCHIVED}"
+      else
+        "#{Project.table_name}.status = #{Project::STATUS_ACTIVE}"
+      end
     if !options[:skip_pre_condition] && perm && perm.project_module
       # If the permission belongs to a project module, make sure the module is enabled
-      base_statement += " AND EXISTS (SELECT 1 AS one FROM #{EnabledModule.table_name} em WHERE em.project_id = #{Project.table_name}.id AND em.name='#{perm.project_module}')"
+      base_statement +=
+        " AND EXISTS (SELECT 1 AS one FROM #{EnabledModule.table_name} em" \
+          " WHERE em.project_id = #{Project.table_name}.id" \
+          " AND em.name='#{perm.project_module}')"
     end
     if project = options[:project]
       project_statement = project.project_condition(options[:with_subprojects])
@@ -193,7 +207,10 @@ class Project < ActiveRecord::Base
           if user.id
             group = role.anonymous? ? Group.anonymous : Group.non_member
             principal_ids = [user.id, group.id].compact
-            s = "(#{s} AND #{Project.table_name}.id NOT IN (SELECT project_id FROM #{Member.table_name} WHERE user_id IN (#{principal_ids.join(',')})))"
+            s =
+              "(#{s} AND #{Project.table_name}.id NOT IN " \
+                "(SELECT project_id FROM #{Member.table_name} " \
+                "WHERE user_id IN (#{principal_ids.join(',')})))"
           end
           statement_by_role[role] = s
         end
@@ -229,11 +246,13 @@ class Project < ActiveRecord::Base
   end
 
   def principals
-    @principals ||= Principal.active.joins(:members).where("#{Member.table_name}.project_id = ?", id).distinct
+    @principals ||=
+      Principal.active.joins(:members).where("#{Member.table_name}.project_id = ?", id).distinct
   end
 
   def users
-    @users ||= User.active.joins(:members).where("#{Member.table_name}.project_id = ?", id).distinct
+    @users ||=
+      User.active.joins(:members).where("#{Member.table_name}.project_id = ?", id).distinct
   end
 
   # Returns the Systemwide and project specific activities
@@ -295,6 +314,15 @@ class Project < ActiveRecord::Base
     end
   end
 
+  # returns the time log activity to be used when logging time via a changeset
+  def commit_logtime_activity
+    activity_id = Setting.commit_logtime_activity_id.to_i
+    if activity_id > 0
+      activities
+        .find_by('id = ? OR parent_id = ?', activity_id, activity_id)
+    end
+  end
+
   # Returns a :conditions SQL string that can be used to find the issues associated with this project.
   #
   # Examples:
@@ -302,14 +330,20 @@ class Project < ActiveRecord::Base
   #   project.project_condition(false) => "projects.id = 1"
   def project_condition(with_subprojects)
     cond = "#{Project.table_name}.id = #{id}"
-    cond = "(#{cond} OR (#{Project.table_name}.lft > #{lft} AND #{Project.table_name}.rgt < #{rgt}))" if with_subprojects
+    if with_subprojects
+      cond = "(#{cond} OR (#{Project.table_name}.lft > #{lft} AND " \
+               "#{Project.table_name}.rgt < #{rgt}))"
+    end
     cond
   end
 
   def self.find(*args)
     if args.first && args.first.is_a?(String) && !/^\d*$/.match?(args.first)
       project = find_by_identifier(*args)
-      raise ActiveRecord::RecordNotFound, "Couldn't find Project with identifier=#{args.first}" if project.nil?
+      if project.nil?
+        raise ActiveRecord::RecordNotFound, "Couldn't find Project with identifier=#{args.first}"
+      end
+
       project
     else
       super
@@ -377,6 +411,7 @@ class Project < ActiveRecord::Base
         exists?
       return false
     end
+
     Project.transaction do
       archive!
     end
@@ -402,6 +437,7 @@ class Project < ActiveRecord::Base
   # by the current user
   def allowed_parents(user=User.current)
     return @allowed_parents if @allowed_parents
+
     @allowed_parents = Project.allowed_to(user, :add_subprojects).to_a
     @allowed_parents = @allowed_parents - self_and_descendants
     if user.allowed_to?(:add_project, nil, :global => true) || (!new_record? && parent.nil?)
@@ -471,7 +507,10 @@ class Project < ActiveRecord::Base
     @rolled_up_versions ||=
       Version.
         joins(:project).
-        where("#{Project.table_name}.lft >= ? AND #{Project.table_name}.rgt <= ? AND #{Project.table_name}.status <> ?", lft, rgt, STATUS_ARCHIVED)
+        where(
+          "#{Project.table_name}.lft >= ? AND #{Project.table_name}.rgt <= ?" \
+          " AND #{Project.table_name}.status <> ?", lft, rgt, STATUS_ARCHIVED
+        )
   end
 
   # Returns a scope of the Versions used by the project
@@ -480,35 +519,52 @@ class Project < ActiveRecord::Base
       Version.
         joins(:project).
         preload(:project).
-        where("#{Project.table_name}.status <> ? AND #{Version.table_name}.sharing = 'system'", STATUS_ARCHIVED)
+        where("#{Project.table_name}.status <> ? AND #{Version.table_name}.sharing = 'system'",
+              STATUS_ARCHIVED)
     else
       @shared_versions ||= begin
         r = root? ? self : root
         Version.
           joins(:project).
           preload(:project).
-          where("#{Project.table_name}.id = #{id}" +
-                  " OR (#{Project.table_name}.status <> #{Project::STATUS_ARCHIVED} AND (" +
-                    " #{Version.table_name}.sharing = 'system'" +
-                    " OR (#{Project.table_name}.lft >= #{r.lft} AND #{Project.table_name}.rgt <= #{r.rgt} AND #{Version.table_name}.sharing = 'tree')" +
-                    " OR (#{Project.table_name}.lft < #{lft} AND #{Project.table_name}.rgt > #{rgt} AND #{Version.table_name}.sharing IN ('hierarchy', 'descendants'))" +
-                    " OR (#{Project.table_name}.lft > #{lft} AND #{Project.table_name}.rgt < #{rgt} AND #{Version.table_name}.sharing = 'hierarchy')" +
-                  "))")
+          where(
+            "#{Project.table_name}.id = #{id}" \
+            " OR (#{Project.table_name}.status <> #{Project::STATUS_ARCHIVED} AND (" \
+            " #{Version.table_name}.sharing = 'system'" \
+            " OR (#{Project.table_name}.lft >= #{r.lft}" \
+            " AND #{Project.table_name}.rgt <= #{r.rgt}" \
+            " AND #{Version.table_name}.sharing = 'tree')" \
+            " OR (#{Project.table_name}.lft < #{lft}" \
+            " AND #{Project.table_name}.rgt > #{rgt}" \
+            " AND #{Version.table_name}.sharing IN ('hierarchy', 'descendants'))" \
+            " OR (#{Project.table_name}.lft > #{lft}" \
+            " AND #{Project.table_name}.rgt < #{rgt}" \
+            " AND #{Version.table_name}.sharing = 'hierarchy')" \
+            "))"
+          )
       end
     end
   end
 
-  # Returns a hash of project users grouped by role
-  def users_by_role
-    members.includes(:user, :roles).inject({}) do |h, m|
+  # Returns a hash of project users/groups grouped by role
+  def principals_by_role
+    memberships.includes(:principal, :roles).inject({}) do |h, m|
       m.roles.each do |r|
         h[r] ||= []
-        h[r] << m.user
+        h[r] << m.principal
       end
       h
     end
   end
 
+  # TODO: Remove this method in Redmine 5.0
+  def members_by_role
+    ActiveSupport::Deprecation.warn(
+      "Project#members_by_role will be removed. Use Project#principals_by_role instead."
+    )
+    principals_by_role
+  end
+
   # Adds user as a project member with the default role
   # Used for when a non-admin user creates a project
   def add_default_member(user)
@@ -527,7 +583,10 @@ class Project < ActiveRecord::Base
   # Deletes all project's members
   def delete_all_members
     me, mr = Member.table_name, MemberRole.table_name
-    self.class.connection.delete("DELETE FROM #{mr} WHERE #{mr}.member_id IN (SELECT #{me}.id FROM #{me} WHERE #{me}.project_id = #{id})")
+    self.class.connection.delete(
+      "DELETE FROM #{mr} WHERE #{mr}.member_id IN (SELECT #{me}.id FROM #{me} " \
+        "WHERE #{me}.project_id = #{id})"
+    )
     Member.where(:project_id => id).delete_all
   end
 
@@ -547,7 +606,10 @@ class Project < ActiveRecord::Base
 
     if tracker
       # Rejects users that cannot the view the tracker
-      roles = Role.where(:assignable => true).select {|role| role.permissions_tracker?(:view_issues, tracker)}
+      roles =
+        Role.where(:assignable => true).select do |role|
+          role.permissions_tracker?(:view_issues, tracker)
+        end
       scope = scope.where(:roles => {:id => roles.map(&:id)})
     end
 
@@ -563,7 +625,12 @@ class Project < ActiveRecord::Base
   # Returns the users that should be notified on project events
   def notified_users
     # TODO: User part should be extracted to User#notify_about?
-    members.preload(:principal).select {|m| m.principal.present? && (m.mail_notification? || m.principal.mail_notification == 'all')}.collect {|m| m.principal}
+    users =
+      members.preload(:principal).select do |m|
+        m.principal.present? &&
+         (m.mail_notification? || m.principal.mail_notification == 'all')
+      end
+    users.collect {|m| m.principal}
   end
 
   # Returns a scope of all custom fields enabled for project issues
@@ -659,12 +726,12 @@ class Project < ActiveRecord::Base
   # progress on it's versions.
   def completed_percent(options={:include_subprojects => false})
     if options.delete(:include_subprojects)
-      total = self_and_descendants.collect(&:completed_percent).sum
+      total = self_and_descendants.sum(&:completed_percent)
 
       total / self_and_descendants.count
     else
       if versions.count > 0
-        total = versions.collect(&:completed_percent).sum
+        total = versions.sum(&:completed_percent)
 
         total / versions.count
       else
@@ -686,6 +753,7 @@ class Project < ActiveRecord::Base
       # No write action allowed on closed projects
       return false
     end
+
     # No action allowed on disabled modules
     if action.is_a? Hash
       allowed_actions.include? "#{action[:controller]}/#{action[:action]}"
@@ -694,6 +762,14 @@ class Project < ActiveRecord::Base
     end
   end
 
+  def deletable?(user = User.current)
+    if user.admin?
+      return true
+    else
+      user.allowed_to?(:delete_project, self) && leaf?
+    end
+  end
+
   # Return the enabled module with the given name
   # or nil if the module is not enabled for the project
   def enabled_module(name)
@@ -709,7 +785,11 @@ class Project < ActiveRecord::Base
   def enabled_module_names=(module_names)
     if module_names && module_names.is_a?(Array)
       module_names = module_names.collect(&:to_s).reject(&:blank?)
-      self.enabled_modules = module_names.collect {|name| enabled_modules.detect {|mod| mod.name == name} || EnabledModule.new(:name => name)}
+      self.enabled_modules =
+        module_names.collect do |name|
+          enabled_modules.detect {|mod| mod.name == name} ||
+            EnabledModule.new(:name => name)
+        end
     else
       enabled_modules.clear
     end
@@ -757,7 +837,7 @@ class Project < ActiveRecord::Base
   safe_attributes(
     'enabled_module_names',
     :if =>
-      lambda {|project, user|
+      lambda do |project, user|
         if project.new_record?
           if user.admin?
             true
@@ -767,7 +847,8 @@ class Project < ActiveRecord::Base
         else
           user.allowed_to?(:select_project_modules, project)
         end
-      })
+      end
+  )
 
   safe_attributes(
     'inherit_members',
@@ -777,8 +858,8 @@ class Project < ActiveRecord::Base
     if attrs.respond_to?(:to_unsafe_hash)
       attrs = attrs.to_unsafe_hash
     end
-
     return unless attrs.is_a?(Hash)
+
     attrs = attrs.deep_dup
 
     @unallowed_parent_id = nil
@@ -846,7 +927,9 @@ class Project < ActiveRecord::Base
         to_be_copied.each do |name|
           send "copy_#{name}", project
         end
-        Redmine::Hook.call_hook(:model_project_copy_before_save, :source_project => project, :destination_project => self)
+        Redmine::Hook.call_hook(:model_project_copy_before_save,
+                                :source_project => project,
+                                :destination_project => self)
         save
       else
         false
@@ -858,7 +941,9 @@ class Project < ActiveRecord::Base
   def self.copy_from(project)
     project = project.is_a?(Project) ? project : Project.find(project)
     # clear unique attributes
-    attributes = project.attributes.dup.except('id', 'name', 'identifier', 'status', 'parent_id', 'lft', 'rgt')
+    attributes =
+      project.attributes.dup.except('id', 'name', 'identifier',
+                                    'status', 'parent_id', 'lft', 'rgt')
     copy = Project.new(attributes)
     copy.enabled_module_names = project.enabled_module_names
     copy.trackers = project.trackers
@@ -923,7 +1008,9 @@ class Project < ActiveRecord::Base
       parent.memberships.each do |parent_member|
         member = Member.find_or_new(self.id, parent_member.user_id)
         parent_member.member_roles.each do |parent_member_role|
-          member.member_roles << MemberRole.new(:role => parent_member_role.role, :inherited_from => parent_member_role.id)
+          member.member_roles <<
+            MemberRole.new(:role => parent_member_role.role,
+                           :inherited_from => parent_member_role.id)
         end
         member.save!
       end
@@ -955,11 +1042,15 @@ class Project < ActiveRecord::Base
       project.wiki.pages.each do |page|
         # Skip pages without content
         next if page.content.nil?
-        new_wiki_content = WikiContent.new(page.content.attributes.dup.except("id", "page_id", "updated_on"))
-        new_wiki_page = WikiPage.new(page.attributes.dup.except("id", "wiki_id", "created_on", "parent_id"))
+
+        new_wiki_content =
+          WikiContent.new(page.content.attributes.dup.except("id", "page_id", "updated_on"))
+        new_wiki_page =
+          WikiPage.new(page.attributes.dup.except("id", "wiki_id", "created_on", "parent_id"))
         new_wiki_page.content = new_wiki_content
         wiki.pages << new_wiki_page
-        new_wiki_page.attachments = page.attachments.map{|attachement| attachement.copy(:container => new_wiki_page)}
+        new_wiki_page.attachments =
+          page.attachments.map{|attachement| attachement.copy(:container => new_wiki_page)}
         wiki_pages_map[page.id] = new_wiki_page
       end
 
@@ -979,7 +1070,8 @@ class Project < ActiveRecord::Base
   def copy_versions(project)
     project.versions.each do |version|
       new_version = Version.new
-      new_version.attributes = version.attributes.dup.except("id", "project_id", "created_on", "updated_on")
+      new_version.attributes =
+        version.attributes.dup.except("id", "project_id", "created_on", "updated_on")
 
       new_version.attachments = version.attachments.map do |attachment|
         attachment.copy(:container => new_version)
@@ -1018,7 +1110,10 @@ class Project < ActiveRecord::Base
       new_issue.project = self
       # Changing project resets the custom field values
       # TODO: handle this in Issue#project=
-      new_issue.custom_field_values = issue.custom_field_values.inject({}) {|h,v| h[v.custom_field_id] = v.value; h}
+      new_issue.custom_field_values = issue.custom_field_values.inject({}) do |h, v|
+        h[v.custom_field_id] = v.value
+        h
+      end
       # Reassign fixed_versions by name, since names are unique per project
       if issue.fixed_version && issue.fixed_version.project == project
         new_issue.fixed_version = self.versions.detect {|v| v.name == issue.fixed_version.name}
@@ -1052,7 +1147,12 @@ class Project < ActiveRecord::Base
 
       self.issues << new_issue
       if new_issue.new_record?
-        logger.info "Project#copy_issues: issue ##{issue.id} could not be copied: #{new_issue.errors.full_messages}" if logger && logger.info?
+        if logger && logger.info?
+          logger.info(
+            "Project#copy_issues: issue ##{issue.id} could not be copied: " \
+              "#{new_issue.errors.full_messages}"
+          )
+        end
       else
         issues_map[issue.id] = new_issue unless new_issue.new_record?
       end
@@ -1074,7 +1174,8 @@ class Project < ActiveRecord::Base
       # Relations
       issue.relations_from.each do |source_relation|
         new_issue_relation = IssueRelation.new
-        new_issue_relation.attributes = source_relation.attributes.dup.except("id", "issue_from_id", "issue_to_id")
+        new_issue_relation.attributes =
+          source_relation.attributes.dup.except("id", "issue_from_id", "issue_to_id")
         new_issue_relation.issue_to = issues_map[source_relation.issue_to_id]
         if new_issue_relation.issue_to.nil? && Setting.cross_project_issue_relations?
           new_issue_relation.issue_to = source_relation.issue_to
@@ -1084,7 +1185,8 @@ class Project < ActiveRecord::Base
 
       issue.relations_to.each do |source_relation|
         new_issue_relation = IssueRelation.new
-        new_issue_relation.attributes = source_relation.attributes.dup.except("id", "issue_from_id", "issue_to_id")
+        new_issue_relation.attributes =
+          source_relation.attributes.dup.except("id", "issue_from_id", "issue_to_id")
         new_issue_relation.issue_from = issues_map[source_relation.issue_from_id]
         if new_issue_relation.issue_from.nil? && Setting.cross_project_issue_relations?
           new_issue_relation.issue_from = source_relation.issue_from
@@ -1108,6 +1210,7 @@ class Project < ActiveRecord::Base
       # inherited roles will be added when copying the group membership
       role_ids = member.member_roles.reject(&:inherited?).collect(&:role_id)
       next if role_ids.empty?
+
       new_member.role_ids = role_ids
       new_member.project = self
       self.members << new_member
@@ -1118,7 +1221,9 @@ class Project < ActiveRecord::Base
   def copy_queries(project)
     project.queries.each do |query|
       new_query = query.class.new
-      new_query.attributes = query.attributes.dup.except("id", "project_id", "sort_criteria", "user_id", "type")
+      new_query.attributes =
+        query.attributes.dup.except("id", "project_id", "sort_criteria",
+                                    "user_id", "type")
       new_query.sort_criteria = query.sort_criteria if query.sort_criteria
       new_query.project = self
       new_query.user_id = query.user_id
@@ -1131,7 +1236,9 @@ class Project < ActiveRecord::Base
   def copy_boards(project)
     project.boards.each do |board|
       new_board = Board.new
-      new_board.attributes = board.attributes.dup.except("id", "project_id", "topics_count", "messages_count", "last_message_id")
+      new_board.attributes =
+        board.attributes.dup.except("id", "project_id", "topics_count",
+                                    "messages_count", "last_message_id")
       new_board.project = self
       self.boards << new_board
     end
@@ -1154,13 +1261,20 @@ class Project < ActiveRecord::Base
 
   def allowed_permissions
     @allowed_permissions ||= begin
-      module_names = enabled_modules.loaded? ? enabled_modules.map(&:name) : enabled_modules.pluck(:name)
+      module_names =
+        if enabled_modules.loaded?
+          enabled_modules.map(&:name)
+        else
+          enabled_modules.pluck(:name)
+        end
       Redmine::AccessControl.modules_permissions(module_names).collect {|p| p.name}
     end
   end
 
   def allowed_actions
-    @actions_allowed ||= allowed_permissions.inject([]) { |actions, permission| actions += Redmine::AccessControl.allowed_actions(permission) }.flatten
+    @actions_allowed ||= allowed_permissions.inject([]) do |actions, permission|
+      actions += Redmine::AccessControl.allowed_actions(permission)
+    end.flatten
   end
 
   # Archives subprojects recursively
