@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'active_record'
 
 module ActiveRecord
@@ -95,7 +97,7 @@ module ActionView
       alias :options_for_select_without_non_empty_blank_option :options_for_select
       def options_for_select(container, selected = nil)
         if container.is_a?(Array)
-          container = container.map {|element| element.blank? ? ["&nbsp;".html_safe, ""] : element}
+          container = container.map {|element| element.presence || ["&nbsp;".html_safe, ""]}
         end
         options_for_select_without_non_empty_blank_option(container, selected)
       end
@@ -125,7 +127,7 @@ ActionMailer::Base.add_delivery_method :tmp_file, DeliveryMethods::TmpFile
 module ActionMailer
   class LogSubscriber < ActiveSupport::LogSubscriber
     def deliver(event)
-      recipients = [:to, :cc, :bcc].inject("") do |s, header|
+      recipients = [:to, :cc, :bcc].inject(+"") do |s, header|
         r = Array.wrap(event.payload[header])
         if r.any?
           s << "\n  #{header}: #{r.join(', ')}"
