@@ -354,22 +354,22 @@ class RedCloth3 < String
 
     # Text markup tags, don't conflict with block tags
     SIMPLE_HTML_TAGS = [
-        'tt', 'b', 'i', 'big', 'small', 'em', 'strong', 'dfn', 'code',
-        'samp', 'kbd', 'var', 'cite', 'abbr', 'acronym', 'a', 'img', 'br',
-        'br', 'map', 'q', 'sub', 'sup', 'span', 'bdo'
+      'tt', 'b', 'i', 'big', 'small', 'em', 'strong', 'dfn', 'code',
+      'samp', 'kbd', 'var', 'cite', 'abbr', 'acronym', 'a', 'img', 'br',
+      'br', 'map', 'q', 'sub', 'sup', 'span', 'bdo'
     ]
 
     QTAGS = [
-        ['**', 'b', :limit],
-        ['*', 'strong', :limit],
-        ['??', 'cite', :limit],
-        ['-', 'del', :limit],
-        ['__', 'i', :limit],
-        ['_', 'em', :limit],
-        ['%', 'span', :limit],
-        ['+', 'ins', :limit],
-        ['^', 'sup', :limit],
-        ['~', 'sub', :limit]
+      ['**', 'b', :limit],
+      ['*', 'strong', :limit],
+      ['??', 'cite', :limit],
+      ['-', 'del', :limit],
+      ['__', 'i', :limit],
+      ['_', 'em', :limit],
+      ['%', 'span', :limit],
+      ['+', 'ins', :limit],
+      ['^', 'sup', :limit],
+      ['~', 'sub', :limit]
     ]
     QTAGS_JOIN = QTAGS.map {|rc, ht, rtype| Regexp::quote rc}.join('|')
 
@@ -636,7 +636,7 @@ class RedCloth3 < String
     end
 
     def lT( text )
-        text =~ /\#$/ ? 'o' : 'u'
+        /\#$/.match?(text) ? 'o' : 'u'
     end
 
     def hard_break( text )
@@ -908,7 +908,7 @@ class RedCloth3 < String
 
     def refs( text )
         @rules.each do |rule_name|
-            method( rule_name ).call( text ) if rule_name.to_s.match /^refs_/
+            method( rule_name ).call( text ) if rule_name.to_s.match? /^refs_/
         end
     end
 
@@ -1016,7 +1016,7 @@ class RedCloth3 < String
     def clean_white_space( text )
         # normalize line breaks
         text.gsub!( /\r\n/, "\n" )
-        text.gsub!( /\r/, "\n" )
+        text.tr!( "\r", "\n" )
         text.gsub!( /\t/, '    ' )
         text.gsub!( /^ +$/, '' )
         text.gsub!( /\n{3,}/, "\n\n" )
@@ -1061,9 +1061,9 @@ class RedCloth3 < String
             text.gsub!( ALLTAG_MATCH ) do |line|
                 ## matches are off if we're between <code>, <pre> etc.
                 if $1
-                    if line =~ OFFTAG_OPEN
+                    if OFFTAG_OPEN.match?(line)
                         codepre += 1
-                    elsif line =~ OFFTAG_CLOSE
+                    elsif OFFTAG_CLOSE.match?(line)
                         codepre -= 1
                         codepre = 0 if codepre < 0
                     end
