@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -26,7 +26,7 @@ class Redmine::ApiTest::MembershipsTest < Redmine::ApiTest::Base
     get '/projects/1/memberships.xml', :headers => credentials('jsmith')
 
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
     assert_select 'memberships[type=array] membership id', :text => '2' do
       assert_select '~ user[id="3"][name="Dave Lopper"]'
       assert_select '~ roles role[id="2"][name=Developer]'
@@ -37,12 +37,13 @@ class Redmine::ApiTest::MembershipsTest < Redmine::ApiTest::Base
     get '/projects/1/memberships.json', :headers => credentials('jsmith')
 
     assert_response :success
-    assert_equal 'application/json', @response.content_type
+    assert_equal 'application/json', @response.media_type
     json = ActiveSupport::JSON.decode(response.body)
     assert_equal 3,  json["total_count"]
     assert_equal 25, json["limit"]
     assert_equal 0,  json["offset"]
-    assert_include({
+    assert_include(
+      {
         "id"=>1,
         "project" => {"name"=>"eCookbook", "id"=>1},
         "roles" => [{"name"=>"Manager", "id"=>1}],
@@ -71,10 +72,11 @@ class Redmine::ApiTest::MembershipsTest < Redmine::ApiTest::Base
 
   test "POST /projects/:project_id/memberships.xml should create the membership" do
     assert_difference 'Member.count' do
-      post '/projects/1/memberships.xml',
-        :params => {:membership => {:user_id => 7, :role_ids => [2,3]}},
+      post(
+        '/projects/1/memberships.xml',
+        :params => {:membership => {:user_id => 7, :role_ids => [2, 3]}},
         :headers => credentials('jsmith')
-
+      )
       assert_response :created
     end
   end
@@ -83,22 +85,24 @@ class Redmine::ApiTest::MembershipsTest < Redmine::ApiTest::Base
     group = Group.find(11)
 
     assert_difference 'Member.count', 1 + group.users.count do
-      post '/projects/1/memberships.xml',
-        :params => {:membership => {:user_id => 11, :role_ids => [2,3]}},
+      post(
+        '/projects/1/memberships.xml',
+        :params => {:membership => {:user_id => 11, :role_ids => [2, 3]}},
         :headers => credentials('jsmith')
-
+      )
       assert_response :created
     end
   end
 
   test "POST /projects/:project_id/memberships.xml with invalid parameters should return errors" do
     assert_no_difference 'Member.count' do
-      post '/projects/1/memberships.xml',
-        :params => {:membership => {:role_ids => [2,3]}},
+      post(
+        '/projects/1/memberships.xml',
+        :params => {:membership => {:role_ids => [2, 3]}},
         :headers => credentials('jsmith')
-
+      )
       assert_response :unprocessable_entity
-      assert_equal 'application/xml', @response.content_type
+      assert_equal 'application/xml', @response.media_type
       assert_select 'errors error', :text => "Principal cannot be blank"
     end
   end
@@ -107,7 +111,7 @@ class Redmine::ApiTest::MembershipsTest < Redmine::ApiTest::Base
     get '/memberships/2.xml', :headers => credentials('jsmith')
 
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
     assert_select 'membership id', :text => '2' do
       assert_select '~ user[id="3"][name="Dave Lopper"]'
       assert_select '~ roles role[id="2"][name=Developer]'
@@ -118,39 +122,44 @@ class Redmine::ApiTest::MembershipsTest < Redmine::ApiTest::Base
     get '/memberships/2.json', :headers => credentials('jsmith')
 
     assert_response :success
-    assert_equal 'application/json', @response.content_type
+    assert_equal 'application/json', @response.media_type
     json = ActiveSupport::JSON.decode(response.body)
     assert_equal(
-      {"membership" => {
-        "id" => 2,
-        "project" => {"name"=>"eCookbook", "id"=>1},
-        "roles" => [{"name"=>"Developer", "id"=>2}],
-        "user" => {"name"=>"Dave Lopper", "id"=>3}}
+      {
+        "membership" => {
+          "id" => 2,
+          "project" => {"name"=>"eCookbook", "id"=>1},
+          "roles" => [{"name"=>"Developer", "id"=>2}],
+          "user" => {"name"=>"Dave Lopper", "id"=>3}
+        }
       },
-      json)
+      json
+    )
   end
 
   test "PUT /memberships/:id.xml should update the membership" do
-    assert_not_equal [1,2], Member.find(2).role_ids.sort
+    assert_not_equal [1, 2], Member.find(2).role_ids.sort
     assert_no_difference 'Member.count' do
-      put '/memberships/2.xml',
-        :params => {:membership => {:user_id => 3, :role_ids => [1,2]}},
+      put(
+        '/memberships/2.xml',
+        :params => {:membership => {:user_id => 3, :role_ids => [1, 2]}},
         :headers => credentials('jsmith')
-
+      )
       assert_response :no_content
       assert_equal '', @response.body
     end
     member = Member.find(2)
-    assert_equal [1,2], member.role_ids.sort
+    assert_equal [1, 2], member.role_ids.sort
   end
 
   test "PUT /memberships/:id.xml with invalid parameters should return errors" do
-    put '/memberships/2.xml',
+    put(
+      '/memberships/2.xml',
       :params => {:membership => {:user_id => 3, :role_ids => [99]}},
       :headers => credentials('jsmith')
-
+    )
     assert_response :unprocessable_entity
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
     assert_select 'errors error', :text => "Role cannot be empty"
   end
 
