@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,12 +20,12 @@
 require File.expand_path('../../../../test_helper', __FILE__)
 
 class Redmine::MimeTypeTest < ActiveSupport::TestCase
-
   def test_of
-    to_test = {'test.txt' => 'text/plain',
-               'test.c' => 'text/x-c',
-               'TEST.JPG' => 'image/jpeg',
-               }
+    to_test = {
+      'test.txt' => 'text/plain',
+      'test.c' => 'text/x-c',
+      'TEST.JPG' => 'image/jpeg',
+    }
     to_test.each do |name, expected|
       assert_equal expected, Redmine::MimeType.of(name)
     end
@@ -36,10 +36,11 @@ class Redmine::MimeTypeTest < ActiveSupport::TestCase
   end
 
   def test_css_class_of
-    to_test = {'test.txt' => 'text-plain',
-               'test.c' => 'text-x-c',
-               'TEST.JPG' => 'image-jpeg',
-               }
+    to_test = {
+      'test.txt' => 'text-plain',
+      'test.c' => 'text-x-c',
+      'TEST.JPG' => 'image-jpeg',
+    }
     to_test.each do |name, expected|
       assert_equal expected, Redmine::MimeType.css_class_of(name)
     end
@@ -50,10 +51,11 @@ class Redmine::MimeTypeTest < ActiveSupport::TestCase
   end
 
   def test_main_mimetype_of
-    to_test = {'test.txt' => 'text',
-               'test.c' => 'text',
-               'TEST.JPG' => 'image',
-               }
+    to_test = {
+      'test.txt' => 'text',
+      'test.c' => 'text',
+      'TEST.JPG' => 'image',
+    }
     to_test.each do |name, expected|
       assert_equal expected, Redmine::MimeType.main_mimetype_of(name)
     end
@@ -64,11 +66,12 @@ class Redmine::MimeTypeTest < ActiveSupport::TestCase
   end
 
   def test_is_type
-    to_test = {['text', 'test.unk'] => false,
-               ['text', 'test.txt'] => true,
-               ['text', 'test.c'] => true,
-               ['image', 'TEST.JPG'] => true,
-               }
+    to_test = {
+      ['text', 'test.unk'] => false,
+      ['text', 'test.txt'] => true,
+      ['text', 'test.c'] => true,
+      ['image', 'TEST.JPG'] => true,
+    }
     to_test.each do |args, expected|
       assert_equal expected, Redmine::MimeType.is_type?(*args)
     end
