@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -68,12 +68,14 @@ class ProjectNestedSetConcurrencyTest < ActiveSupport::TestCase
   # Generates a bare project with random name
   # and identifier
   def generate_project!(attributes={})
-    identifier = "a"+Redmine::Utils.random_hex(6)
-    Project.generate!({
+    identifier = "a" + Redmine::Utils.random_hex(6)
+    Project.generate!(
+      {
         :identifier => identifier,
         :name => identifier,
         :tracker_ids => [],
         :enabled_module_names => []
-      }.merge(attributes))
+      }.merge(attributes)
+    )
   end
 end
