@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -153,13 +153,13 @@ class ProjectTest < ActiveSupport::TestCase
 
   def test_members_should_be_active_users
     Project.all.each do |project|
-      assert_nil project.members.detect {|m| !(m.user.is_a?(User) && m.user.active?) }
+      assert_nil project.members.detect {|m| !(m.user.is_a?(User) && m.user.active?)}
     end
   end
 
   def test_users_should_be_active_users
     Project.all.each do |project|
-      assert_nil project.users.detect {|u| !(u.is_a?(User) && u.active?) }
+      assert_nil project.users.detect {|u| !(u.is_a?(User) && u.active?)}
     end
   end
 
@@ -235,15 +235,19 @@ class ProjectTest < ActiveSupport::TestCase
 
     @ecookbook.destroy
     # make sure that the project non longer exists
-    assert_raise(ActiveRecord::RecordNotFound) { Project.find(@ecookbook.id) }
+    assert_raise(ActiveRecord::RecordNotFound) {Project.find(@ecookbook.id)}
     # make sure related data was removed
-    assert_nil Member.where(:project_id => @ecookbook.id).first
-    assert_nil Board.where(:project_id => @ecookbook.id).first
-    assert_nil Issue.where(:project_id => @ecookbook.id).first
+    assert_not Member.where(:project_id => @ecookbook.id).exists?
+    assert_not Board.where(:project_id => @ecookbook.id).exists?
+    assert_not Issue.where(:project_id => @ecookbook.id).exists?
   end
 
   def test_destroy_should_destroy_subtasks
-    issues = (0..2).to_a.map {Issue.create!(:project_id => 1, :tracker_id => 1, :author_id => 1, :subject => 'test')}
+    issues =
+      (0..2).to_a.map do
+        Issue.create!(:project_id => 1, :tracker_id => 1,
+                      :author_id => 1, :subject => 'test')
+      end
     issues[0].update! :parent_issue_id => issues[1].id
     issues[2].update! :parent_issue_id => issues[1].id
     assert_equal 2, issues[1].children.count
@@ -370,9 +374,13 @@ class ProjectTest < ActiveSupport::TestCase
     issue_with_local_fixed_version.reload
     issue_with_hierarchy_fixed_version.reload
 
-    assert_equal 4, issue_with_local_fixed_version.fixed_version_id, "Fixed version was not keep on an issue local to the moved project"
-    assert_nil issue_with_hierarchy_fixed_version.fixed_version_id, "Fixed version is still set after moving the Project out of the hierarchy where the version is defined in"
-    assert_nil parent_issue.fixed_version_id, "Fixed version is still set after moving the Version out of the hierarchy for the issue."
+    assert_equal 4, issue_with_local_fixed_version.fixed_version_id,
+                 "Fixed version was not keep on an issue local to the moved project"
+    assert_nil issue_with_hierarchy_fixed_version.fixed_version_id,
+               "Fixed version is still set after moving the Project out of the hierarchy " \
+                 "where the version is defined in"
+    assert_nil parent_issue.fixed_version_id,
+               "Fixed version is still set after moving the Version out of the hierarchy for the issue."
   end
 
   def test_parent
@@ -455,19 +463,23 @@ class ProjectTest < ActiveSupport::TestCase
     assert Project.find(3).allowed_parents.include?(nil)
   end
 
-  def test_users_by_role
-    users_by_role = Project.find(1).users_by_role
-    assert_kind_of Hash, users_by_role
+  def test_principals_by_role
+    group = Group.find(10)
+    Member.create!(:principal => group, :project_id => 1, :role_ids => [1])
+
+    principals_by_role = Project.find(1).principals_by_role
+    assert_kind_of Hash, principals_by_role
     role = Role.find(1)
-    assert_kind_of Array, users_by_role[role]
-    assert users_by_role[role].include?(User.find(2))
+    assert_kind_of Array, principals_by_role[role]
+    assert principals_by_role[role].include?(User.find(2))
+    assert principals_by_role[role].include?(group)
   end
 
   def test_rolled_up_trackers
     parent = Project.find(1)
-    parent.trackers = Tracker.find([1,2])
+    parent.trackers = Tracker.find([1, 2])
     child = parent.children.find(3)
-    child.trackers = Tracker.find([2,3])
+    child.trackers = Tracker.find([2, 3])
 
     assert_equal [1, 2], parent.tracker_ids
     assert_equal [2, 3], child.trackers.collect(&:id)
@@ -481,12 +493,12 @@ class ProjectTest < ActiveSupport::TestCase
 
   def test_rolled_up_trackers_should_ignore_archived_subprojects
     parent = Project.find(1)
-    parent.trackers = Tracker.find([1,2])
+    parent.trackers = Tracker.find([1, 2])
     child = parent.children.find(3)
-    child.trackers = Tracker.find([1,3])
+    child.trackers = Tracker.find([1, 3])
     parent.children.each(&:archive)
 
-    assert_equal [1,2], parent.rolled_up_trackers.collect(&:id)
+    assert_equal [1, 2], parent.rolled_up_trackers.collect(&:id)
   end
 
   def test_rolled_up_statuses
@@ -618,7 +630,7 @@ class ProjectTest < ActiveSupport::TestCase
     child = parent.children.find(3)
     private_child = parent.children.find(5)
 
-    assert_equal [1,2,3], parent.version_ids.sort
+    assert_equal [1, 2, 3], parent.version_ids.sort
     assert_equal [4], child.version_ids
     assert_equal [6], private_child.version_ids
     assert_equal [7], Version.where(:sharing => 'system').collect(&:id)
@@ -628,7 +640,7 @@ class ProjectTest < ActiveSupport::TestCase
       assert_kind_of Version, version
     end
 
-    assert_equal [1,2,3,4,6,7], parent.shared_versions.collect(&:id).sort
+    assert_equal [1, 2, 3, 4, 6, 7], parent.shared_versions.collect(&:id).sort
   end
 
   def test_shared_versions_should_ignore_archived_subprojects
@@ -637,7 +649,7 @@ class ProjectTest < ActiveSupport::TestCase
     child.archive
     parent.reload
 
-    assert_equal [1,2,3], parent.version_ids.sort
+    assert_equal [1, 2, 3], parent.version_ids.sort
     assert_equal [4], child.version_ids
     assert !parent.shared_versions.collect(&:id).include?(4)
   end
@@ -647,7 +659,7 @@ class ProjectTest < ActiveSupport::TestCase
     parent = Project.find(1)
     child = parent.children.find(5)
 
-    assert_equal [1,2,3], parent.version_ids.sort
+    assert_equal [1, 2, 3], parent.version_ids.sort
     assert_equal [6], child.version_ids
 
     versions = parent.shared_versions.visible(user)
@@ -813,11 +825,14 @@ class ProjectTest < ActiveSupport::TestCase
     overridden_activity = TimeEntryActivity.new({:name => "Project", :project => Project.find(2)})
     assert overridden_activity.save!
 
-    assert !project.activities.include?(overridden_activity), "Project specific Activity found on a different project"
+    assert !project.activities.include?(overridden_activity),
+           "Project specific Activity found on a different project"
   end
 
   def test_activities_should_handle_nils
-    overridden_activity = TimeEntryActivity.new({:name => "Project", :project => Project.find(1), :parent => TimeEntryActivity.first})
+    overridden_activity =
+      TimeEntryActivity.new({:name => "Project", :project => Project.find(1),
+                             :parent => TimeEntryActivity.first})
     TimeEntryActivity.delete_all
 
     # No activities
@@ -833,16 +848,24 @@ class ProjectTest < ActiveSupport::TestCase
   def test_activities_should_override_system_activities_with_project_activities
     project = Project.find(1)
     parent_activity = TimeEntryActivity.first
-    overridden_activity = TimeEntryActivity.new({:name => "Project", :project => project, :parent => parent_activity})
+    overridden_activity =
+      TimeEntryActivity.new({:name => "Project", :project => project,
+                             :parent => parent_activity})
     assert overridden_activity.save!
 
-    assert project.activities.include?(overridden_activity), "Project specific Activity not found"
-    assert !project.activities.include?(parent_activity), "System Activity found when it should have been overridden"
+    assert project.activities.include?(overridden_activity),
+           "Project specific Activity not found"
+    assert !project.activities.include?(parent_activity),
+           "System Activity found when it should have been overridden"
   end
 
   def test_activities_should_include_inactive_activities_if_specified
     project = Project.find(1)
-    overridden_activity = TimeEntryActivity.new({:name => "Project", :project => project, :parent => TimeEntryActivity.first, :active => false})
+    overridden_activity =
+      TimeEntryActivity.new(
+        {:name => "Project", :project => project,
+         :parent => TimeEntryActivity.first, :active => false}
+      )
     assert overridden_activity.save!
 
     assert project.activities(true).include?(overridden_activity), "Inactive Project specific Activity not found"
@@ -852,11 +875,15 @@ class ProjectTest < ActiveSupport::TestCase
     project = Project.find(1)
     system_activity = TimeEntryActivity.find_by_name('Design')
     assert system_activity.active?
-    overridden_activity = TimeEntryActivity.create!(:name => "Project", :project => project, :parent => system_activity, :active => false)
+    overridden_activity =
+      TimeEntryActivity.create!(:name => "Project", :project => project,
+                                :parent => system_activity, :active => false)
     assert overridden_activity.save!
 
-    assert !project.activities.include?(overridden_activity), "Inactive Project specific Activity not found"
-    assert !project.activities.include?(system_activity), "System activity found when the project has an inactive override"
+    assert !project.activities.include?(overridden_activity),
+           "Inactive Project specific Activity not found"
+    assert !project.activities.include?(system_activity),
+           "System activity found when the project has an inactive override"
   end
 
   def test_close_completed_versions
@@ -954,9 +981,11 @@ class ProjectTest < ActiveSupport::TestCase
     project = Project.generate!
     project.trackers << Tracker.generate!
     v1 = Version.generate!(:project => project)
-    Issue.generate!(:project => project, :status => IssueStatus.find_by_name('Closed'), :fixed_version => v1)
+    Issue.generate!(:project => project, :status => IssueStatus.find_by_name('Closed'),
+                    :fixed_version => v1)
     v2 = Version.generate!(:project => project)
-    Issue.generate!(:project => project, :status => IssueStatus.find_by_name('Closed'), :fixed_version => v2)
+    Issue.generate!(:project => project, :status => IssueStatus.find_by_name('Closed'),
+                    :fixed_version => v2)
 
     assert_equal 100, project.completed_percent
   end
@@ -965,9 +994,11 @@ class ProjectTest < ActiveSupport::TestCase
     project = Project.generate!
     project.trackers << Tracker.generate!
     v1 = Version.generate!(:project => project)
-    Issue.generate!(:project => project, :status => IssueStatus.find_by_name('New'), :estimated_hours => 10, :done_ratio => 50, :fixed_version => v1)
+    Issue.generate!(:project => project, :status => IssueStatus.find_by_name('New'),
+                    :estimated_hours => 10, :done_ratio => 50, :fixed_version => v1)
     v2 = Version.generate!(:project => project)
-    Issue.generate!(:project => project, :status => IssueStatus.find_by_name('New'), :estimated_hours => 10, :done_ratio => 50, :fixed_version => v2)
+    Issue.generate!(:project => project, :status => IssueStatus.find_by_name('New'),
+                    :estimated_hours => 10, :done_ratio => 50, :fixed_version => v2)
 
     assert_equal 50, project.completed_percent
   end
@@ -977,7 +1008,8 @@ class ProjectTest < ActiveSupport::TestCase
     role = Role.generate!
 
     user_with_membership_notification = User.generate!(:mail_notification => 'selected')
-    Member.create!(:project => project, :roles => [role], :principal => user_with_membership_notification, :mail_notification => true)
+    Member.create!(:project => project, :roles => [role],
+                   :principal => user_with_membership_notification, :mail_notification => true)
 
     all_events_user = User.generate!(:mail_notification => 'all')
     Member.create!(:project => project, :roles => [role], :principal => all_events_user)
@@ -994,12 +1026,18 @@ class ProjectTest < ActiveSupport::TestCase
     only_owned_user = User.generate!(:mail_notification => 'only_owner')
     Member.create!(:project => project, :roles => [role], :principal => only_owned_user)
 
-    assert project.notified_users.include?(user_with_membership_notification), "should include members with a mail notification"
-    assert project.notified_users.include?(all_events_user), "should include users with the 'all' notification option"
-    assert !project.notified_users.include?(no_events_user), "should not include users with the 'none' notification option"
-    assert !project.notified_users.include?(only_my_events_user), "should not include users with the 'only_my_events' notification option"
-    assert !project.notified_users.include?(only_assigned_user), "should not include users with the 'only_assigned' notification option"
-    assert !project.notified_users.include?(only_owned_user), "should not include users with the 'only_owner' notification option"
+    assert project.notified_users.include?(user_with_membership_notification),
+           "should include members with a mail notification"
+    assert project.notified_users.include?(all_events_user),
+           "should include users with the 'all' notification option"
+    assert !project.notified_users.include?(no_events_user),
+           "should not include users with the 'none' notification option"
+    assert !project.notified_users.include?(only_my_events_user),
+           "should not include users with the 'only_my_events' notification option"
+    assert !project.notified_users.include?(only_assigned_user),
+           "should not include users with the 'only_assigned' notification option"
+    assert !project.notified_users.include?(only_owned_user),
+           "should not include users with the 'only_owner' notification option"
   end
 
   def test_override_roles_without_builtin_group_memberships
@@ -1038,7 +1076,8 @@ class ProjectTest < ActiveSupport::TestCase
 
   def test_combination_of_visible_and_distinct_scopes_in_case_anonymous_group_has_memberships_should_not_error
     project = Project.find(1)
-    member = Member.create!(:project => project, :principal => Group.anonymous, :roles => [Role.generate!])
+    member = Member.create!(:project => project, :principal => Group.anonymous,
+                            :roles => [Role.generate!])
     project.members << member
     assert_nothing_raised do
       Project.distinct.visible.to_a
@@ -1055,16 +1094,29 @@ class ProjectTest < ActiveSupport::TestCase
     user = User.find(2)
     project = Project.find(1)
 
-    project.send :safe_attributes=, {'custom_field_values' => {
-                                       cf1.id.to_s => 'value1', cf2.id.to_s => 'value2'
-                                     }}, user
+    project.send(
+      :safe_attributes=,
+      {
+        'custom_field_values' => {
+          cf1.id.to_s => 'value1', cf2.id.to_s => 'value2'
+        }
+      },
+      user
+    )
     assert_equal 'value1', project.custom_field_value(cf1)
     assert_nil project.custom_field_value(cf2)
 
-    project.send :safe_attributes=, {'custom_fields' => [
-                                      {'id' => cf1.id.to_s, 'value' => 'valuea'},
-                                      {'id' => cf2.id.to_s, 'value' => 'valueb'}
-                                    ]}, user
+    project.send(
+      :safe_attributes=,
+      {
+        'custom_fields' =>
+          [
+            {'id' => cf1.id.to_s, 'value' => 'valuea'},
+            {'id' => cf2.id.to_s, 'value' => 'valueb'}
+          ]
+      },
+      user
+    )
     assert_equal 'valuea', project.custom_field_value(cf1)
     assert_nil project.custom_field_value(cf2)
   end
