@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,25 +25,27 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
   include Redmine::I18n
 
   REPOSITORY_PATH = Rails.root.join('tmp/test/mercurial_repository').to_s
-  NUM_REV = 34
+  NUM_REV = 40
 
   def setup
     User.current = nil
     @project    = Project.find(3)
-    @repository = Repository::Mercurial.create(
-                      :project => @project,
-                      :url     => REPOSITORY_PATH,
-                      :path_encoding => 'ISO-8859-1'
-                      )
+    @repository =
+      Repository::Mercurial.create(
+        :project => @project,
+        :url     => REPOSITORY_PATH,
+        :path_encoding => 'ISO-8859-1'
+      )
     assert @repository
   end
 
   def test_blank_path_to_repository_error_message
     set_language_if_valid 'en'
-    repo = Repository::Mercurial.new(
-                          :project      => @project,
-                          :identifier   => 'test'
-                        )
+    repo =
+      Repository::Mercurial.new(
+        :project      => @project,
+        :identifier   => 'test'
+      )
     assert !repo.save
     assert_include "Path to repository cannot be blank",
                    repo.errors.full_messages
@@ -51,12 +53,13 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
 
   def test_blank_path_to_repository_error_message_fr
     set_language_if_valid 'fr'
-    repo = Repository::Mercurial.new(
-                          :project      => @project,
-                          :url          => "",
-                          :identifier   => 'test',
-                          :path_encoding => ''
-                        )
+    repo =
+      Repository::Mercurial.new(
+        :project      => @project,
+        :url          => "",
+        :identifier   => 'test',
+        :path_encoding => ''
+      )
     assert !repo.save
     assert_include 'Chemin du dépôt doit être renseigné(e)', repo.errors.full_messages
   end
@@ -167,7 +170,7 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      assert_equal 46, @repository.filechanges.count
+      assert_equal 51, @repository.filechanges.count
       rev0 = @repository.changesets.find_by_revision('0')
       assert_equal "Initial import.\nThe repository contains 3 files.",
                    rev0.comments
@@ -256,21 +259,25 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
       assert_equal [(NUM_REV - 1).to_s, (NUM_REV - 2).to_s], changesets.collect(&:revision)
 
       # with_filepath
-      changesets = @repository.latest_changesets(
-                      '/sql_escape/percent%dir/percent%file1.txt', nil)
+      changesets =
+        @repository.latest_changesets(
+          '/sql_escape/percent%dir/percent%file1.txt', nil
+        )
       assert_equal %w|30 11 10 9|, changesets.collect(&:revision)
 
-      changesets = @repository.latest_changesets(
-                      '/sql_escape/underscore_dir/understrike_file.txt', nil)
+      changesets =
+        @repository.latest_changesets(
+          '/sql_escape/underscore_dir/understrike_file.txt', nil
+        )
       assert_equal %w|30 12 9|, changesets.collect(&:revision)
 
       changesets = @repository.latest_changesets('README', nil)
       assert_equal %w|31 30 28 17 8 6 1 0|, changesets.collect(&:revision)
 
-      changesets = @repository.latest_changesets('README','8')
+      changesets = @repository.latest_changesets('README', '8')
       assert_equal %w|8 6 1 0|, changesets.collect(&:revision)
 
-      changesets = @repository.latest_changesets('README','8', 2)
+      changesets = @repository.latest_changesets('README', '8', 2)
       assert_equal %w|8 6|, changesets.collect(&:revision)
 
       # with_dirpath
@@ -580,7 +587,7 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
       %w|27 7bbf4c738e71 7bbf|.each do |r2|
         changeset = @repository.find_changeset_by_name(r2)
         %w|28 3ae45e2d177d 3ae45|.each do |r1|
-        assert_equal @repository.find_changeset_by_name(r1), changeset.next
+          assert_equal @repository.find_changeset_by_name(r1), changeset.next
         end
       end
     end
@@ -590,7 +597,7 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      [(NUM_REV - 1).to_s, "2e6d54642923", "2e6d5"].each do |r1|
+      [(NUM_REV - 1).to_s, "04aed9840e92", "04aed"].each do |r1|
         changeset = @repository.find_changeset_by_name(r1)
         assert_nil changeset.next
       end
